@@ -23,8 +23,8 @@
 #include <sybdb.h>
 
 
-static char rcsid_sqldb_h[] = "$Id: sqldb.h,v 1.2 2002/11/21 16:53:43 freddy77 Exp $";
-static void *no_unused_sqldb_h_warn[] = { rcsid_sqldb_h, no_unused_sqldb_h_warn };
+static const char rcsid_sqldb_h[] = "$Id: sqldb.h,v 1.3 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_sqldb_h_warn[] = { rcsid_sqldb_h, no_unused_sqldb_h_warn };
 
 
 #endif
