@@ -20,11 +20,15 @@
 #ifndef _tds_configs_h_
 #define _tds_configs_h_
 
-static char rcsid_tds_configs_h[] = "$Id: tds_configs.h,v 1.18 2002/11/21 16:53:43 freddy77 Exp $";
-static void *no_unused_tds_configs_h_warn[] = { rcsid_tds_configs_h, no_unused_tds_configs_h_warn };
+static const char rcsid_tds_configs_h[] = "$Id: tds_configs.h,v 1.20 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_tds_configs_h_warn[] = { rcsid_tds_configs_h, no_unused_tds_configs_h_warn };
 
 #include "freetds_sysconfdir.h"
 
+#ifndef _tds_h_
+#error tds.h must be included before tds_configs.h
+#endif
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -33,9 +37,9 @@ extern "C"
 #endif
 #endif
 
-#define FREETDS_SYSCONFFILE FREETDS_SYSCONFDIR "/freetds.conf"
-#define FREETDS_POOLCONFFILE FREETDS_SYSCONFDIR "/pool.conf"
-#define FREETDS_LOCALECONFFILE FREETDS_SYSCONFDIR "/locales.conf"
+#define FREETDS_SYSCONFFILE FREETDS_SYSCONFDIR TDS_SDIR_SEPARATOR "freetds.conf"
+#define FREETDS_POOLCONFFILE FREETDS_SYSCONFDIR TDS_SDIR_SEPARATOR "pool.conf"
+#define FREETDS_LOCALECONFFILE FREETDS_SYSCONFDIR TDS_SDIR_SEPARATOR "locales.conf"
 
 #ifdef __cplusplus
 #if 0
