@@ -1,4 +1,4 @@
-static char rcsid_freebcp_h[] = "$Id: freebcp.h,v 1.4 2002/12/02 21:56:43 jklowden Exp $";
+static char rcsid_freebcp_h[] = "$Id: freebcp.h,v 1.9 2004/06/19 05:56:29 jklowden Exp $";
 static void *no_unused_freebcp_h_warn[] = { rcsid_freebcp_h, no_unused_freebcp_h_warn };
 
 enum states
@@ -21,7 +21,7 @@ enum states
 	GET_LANGUAGE,
 	GET_PACKETSIZE,
 	GET_CLIENTCHARSET,
-	GET_TESTSIZE,
+	GET_TEXTSIZE,
 	GET_SYBASEDIR,
 	GET_FROMLABEL,
 	GET_TOLABEL,
@@ -31,21 +31,24 @@ enum states
 typedef struct pd
 {
 	char *dbobject;
-	char dbdirection[4];
+	char dbdirection[10];
+	DBINT direction;
 	char hostfilename[FILENAME_MAX + 1];
 	char formatfile[FILENAME_MAX + 1];
-	char errorfile[FILENAME_MAX + 1];
+	char *errorfile;
 	char interfacesfile[FILENAME_MAX + 1];
 	int firstrow;
 	int lastrow;
 	int batchsize;
 	int maxerrors;
+	int textsize;
 	char *fieldterm;
 	char *rowterm;
 	char *user;
 	char *pass;
 	char *server;
 	char *hint;
+	int packetsize;
 	int mflag;
 	int fflag;
 	int eflag;
@@ -60,5 +63,8 @@ typedef struct pd
 	int Iflag;
 	int Sflag;
 	int Pflag;
+	int Tflag;
+	int Aflag;
+	int Eflag;
 }
 PARAMDATA;
