@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-2002 Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,12 +33,17 @@
 #endif
 
 #include <stdio.h>
+#include <assert.h>
 
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif /* HAVE_READLINE */
 
+#if HAVE_ERRNO_H
+#include <errno.h>
+#endif /* HAVE_ERRNO_H */
+
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -62,22 +67,27 @@
 #include "tds.h"
 #include "tdsconvert.h"
 
-static char software_version[] = "$Id: tsql.c,v 1.49.2.2 2003/02/20 20:14:21 freddy77 Exp $";
+static char software_version[] = "$Id: tsql.c,v 1.70 2004/06/28 05:24:56 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 enum
 {
-	OPT_VERSION = 0x01,
-	OPT_TIMER = 0x02,
+	OPT_VERSION =  0x01,
+	OPT_TIMER =    0x02,
 	OPT_NOFOOTER = 0x04,
-	OPT_NOHEADER = 0x08
+	OPT_NOHEADER = 0x08,
+	OPT_QUIET =    0x10
 };
 
+static int global_opt_flags = 0;
+#define QUIET (global_opt_flags & OPT_QUIET)
+
 int do_query(TDSSOCKET * tds, char *buf, int opt_flags);
-void print_usage(char *progname);
+static void tsql_print_usage(const char *progname);
 int get_opt_flags(char *s, int *opt_flags);
 void populate_login(TDSLOGIN * login, int argc, char **argv);
-int tsql_handle_message(TDSCONTEXT * context, TDSSOCKET * tds, TDSMSGINFO * msg);
+int tsql_handle_message(TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg);
+void slurp_input_file(char *fname, char **mybuf, int *bufsz, int *line);
 
 #ifndef HAVE_READLINE
 char *readline(char *prompt);
@@ -86,23 +96,22 @@ void add_history(const char *s);
 char *
 readline(char *prompt)
 {
-	char *buf, line[1000];
+	char line[1000];
 	int i = 0;
 
-	printf("%s", prompt);
+	if (!QUIET)
+	    printf("%s", prompt);
 	if (fgets(line, 1000, stdin) == NULL) {
 		return NULL;
 	}
-	for (i = strlen(line); i > 0; i--) {
+	for (i = strlen(line); i > 0; --i) {
 		if (line[i] == '\n') {
 			line[i] = '\0';
 			break;
 		}
 	}
-	buf = (char *) malloc(strlen(line) + 1);
-	strcpy(buf, line);
 
-	return buf;
+	return strdup(line);
 }
 
 void
@@ -116,7 +125,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 {
 	int rows = 0;
 	int rc, i;
-	TDSCOLINFO *col;
+	TDSCOLUMN *col;
 	int ctype;
 	CONV_RESULT dres;
 	unsigned char *src;
@@ -134,7 +143,7 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 		return 1;
 	}
 
-	while ((rc = tds_process_result_tokens(tds, &resulttype)) == TDS_SUCCEED) {
+	while ((rc = tds_process_result_tokens(tds, &resulttype, NULL)) == TDS_SUCCEED) {
 		if (opt_flags & OPT_TIMER) {
 			gettimeofday(&start, NULL);
 			print_rows = 0;
@@ -167,11 +176,11 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 
 					src = &(tds->res_info->current_row[col->column_offset]);
 					if (is_blob_type(col->column_type))
-						src = (unsigned char*) ((TDSBLOBINFO *) src)->textvalue;
+						src = (unsigned char *) ((TDSBLOB *) src)->textvalue;
 					srclen = col->column_cur_size;
 
 
-					if (tds_convert(tds->tds_ctx, ctype, (TDS_CHAR*) src, srclen, SYBVARCHAR, &dres) < 0)
+					if (tds_convert(tds->tds_ctx, ctype, (TDS_CHAR *) src, srclen, SYBVARCHAR, &dres) < 0)
 						continue;
 					if (print_rows)
 						fprintf(stdout, "%s\t", dres.c);
@@ -190,8 +199,8 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 		}
 
 		if (opt_flags & OPT_VERSION) {
-	char version[64];
-	int line = 0;
+			char version[64];
+			int line = 0;
 
 			line = tds_version(tds, version);
 			if (line) {
@@ -209,11 +218,12 @@ do_query(TDSSOCKET * tds, char *buf, int opt_flags)
 	return 0;
 }
 
-void
-print_usage(char *progname)
+static void
+tsql_print_usage(const char *progname)
 {
-	fprintf(stderr, "Usage: %s [-S <server> | -H <hostname> -p <port>] -U <username> [ -P <password> ] [ -I <config file> ]\n",
-		progname);
+	fprintf(stderr,
+		"Usage:\t%s [-S <server> | -H <hostname> -p <port>] -U <username> [ -P <password> ] [ -I <config file> ]\n\t%s -C\n",
+		progname, progname);
 }
 
 int
@@ -224,17 +234,20 @@ get_opt_flags(char *s, int *opt_flags)
 	int opt;
 
 	/* make sure we have enough elements */
-	argv = (char**) malloc(sizeof(char *) * strlen(s));
+	assert(s && opt_flags);
+	argv = (char **) calloc(strlen(s) + 2, sizeof(char*));
 	if (!argv)
 		return 0;
 
 	/* parse the command line and assign to argv */
+	argv[argc++] = "tsql";
 	argv[argc++] = strtok(s, " ");
-	if (argv[0])
+	if (argv[argc - 1])
 		while ((argv[argc++] = strtok(NULL, " ")) != NULL);
 
+	*opt_flags = 0;
 	optind = 0;		/* reset getopt */
-	while ((opt = getopt(argc - 1, argv, "fhtv")) != -1) {
+	while ((opt = getopt(argc - 1, argv, "fhqtv")) != -1) {
 		switch (opt) {
 		case 'f':
 			*opt_flags |= OPT_NOFOOTER;
@@ -248,6 +261,9 @@ get_opt_flags(char *s, int *opt_flags)
 		case 'v':
 			*opt_flags |= OPT_VERSION;
 			break;
+		case 'q':
+			*opt_flags |= OPT_QUIET;
+			break;
 		}
 	}
 	free(argv);
@@ -257,6 +273,7 @@ get_opt_flags(char *s, int *opt_flags)
 void
 populate_login(TDSLOGIN * login, int argc, char **argv)
 {
+	const TDS_COMPILETIME_SETTINGS *settings;
 	char *hostname = NULL;
 	char *servername = NULL;
 	char *username = NULL;
@@ -266,24 +283,25 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 	int opt;
 	const char *locale = NULL;
 	char *charset = NULL;
+	char *opt_flags_str = NULL;
 
 	setlocale(LC_ALL, "");
 	locale = setlocale(LC_ALL, NULL);
-#ifdef CODESET
-	charset = nl_langinfo(CODESET);
+
+#if HAVE_LOCALE_CHARSET
+	charset = locale_charset();
+#endif
+#if HAVE_NL_LANGINFO && defined(CODESET)
+	if (!charset)
+		charset = nl_langinfo(CODESET);
 #endif
 
-	if (locale)
-		printf("locale is \"%s\"\n", locale);
-	if (charset) {
-		printf("charset is \"%s\"\n", charset);
-	} else {
-		charset = "iso_1";
-		printf("using default charset \"%s\"\n", charset);
-	}
 
-	while ((opt = getopt(argc, argv, "H:S:I:V::P:U:p:v")) != -1) {
+	while ((opt = getopt(argc, argv, "H:S:I:V::P:U:p:vCo:")) != -1) {
 		switch (opt) {
+		case 'o':
+			opt_flags_str = optarg;
+			break;
 		case 'H':
 			hostname = (char *) malloc(strlen(optarg) + 1);
 			strcpy(hostname, optarg);
@@ -307,35 +325,71 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 		case 'p':
 			port = atoi(optarg);
 			break;
+		case 'C':
+			settings = tds_get_compiletime_settings();
+			printf("%s\n%35s %s\n%35s %s\n%35s %s\n%35s %s\n%35s %s\n%35s %s\n%35s %s\n%35s %s\n",
+			       "Compile-time settings (established with the \"configure\" script):",
+			       "Version:", settings->freetds_version,
+			       /* settings->last_update */
+			       "MS db-lib source compatibility:", settings->msdblib ? "yes" : "no",
+			       "Sybase binary compatibility:",
+			       (settings->sybase_compat == -1 ? "unknown" : (settings->sybase_compat ? "yes" : "no")),
+			       "Thread safety:", settings->threadsafe ? "yes" : "no",
+			       "iconv library:", settings->libiconv ? "yes" : "no",
+			       "TDS version:", settings->tdsver,
+			       "iODBC:", settings->iodbc ? "yes" : "no", "unixodbc:", settings->unixodbc ? "yes" : "no");
+			exit(0);
+			break;
 		default:
-			print_usage(argv[0]);
+			tsql_print_usage(argv[0]);
 			exit(1);
 			break;
 		}
 	}
 
+	if (opt_flags_str != NULL) {
+		char *minus_flags = malloc(strlen(opt_flags_str) + 2);
+		if (minus_flags != NULL) {
+			*minus_flags = '-';
+			strcpy(&minus_flags[1], opt_flags_str);
+			get_opt_flags(minus_flags, &global_opt_flags);
+			free(minus_flags);
+		}
+	}
+
+
+	if (locale)
+		if (!QUIET) printf("locale is \"%s\"\n", locale);
+	if (charset) {
+		if (!QUIET) printf("locale charset is \"%s\"\n", charset);
+	} else {
+		charset = "ISO-8859-1";
+		if (!QUIET) printf("using default charset \"%s\"\n", charset);
+	}
+
 	/* validate parameters */
 	if (!servername && !hostname) {
 		fprintf(stderr, "Missing argument -S or -H\n");
-		print_usage(argv[0]);
+		tsql_print_usage(argv[0]);
 		exit(1);
 	}
 	if (hostname && !port) {
 		fprintf(stderr, "Missing argument -p \n");
-		print_usage(argv[0]);
+		tsql_print_usage(argv[0]);
 		exit(1);
 	}
 	if (!username) {
 		fprintf(stderr, "Missing argument -U \n");
-		print_usage(argv[0]);
+		tsql_print_usage(argv[0]);
 		exit(1);
 	}
 	if (!servername && !hostname) {
-		print_usage(argv[0]);
+		tsql_print_usage(argv[0]);
 		exit(1);
 	}
 	if (!password) {
-	char *tmp = getpass("Password: ");
+		/* TODO some platoforms accept only short password (ie. 8 chars in HP/UX 10.20) */
+		char *tmp = getpass("Password: ");
 
 		password = strdup(tmp);
 	}
@@ -349,9 +403,8 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 		tds_set_app(login, "TSQL");
 		tds_set_library(login, "TDS-Library");
 		tds_set_server(login, servername);
-		tds_set_charset(login, charset);
+		tds_set_client_charset(login, charset);
 		tds_set_language(login, "us_english");
-		tds_set_packet(login, 512);
 		tds_set_passwd(login, password);
 		if (confile) {
 			tds_set_interfaces_file_loc(confile);
@@ -363,9 +416,8 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 		tds_set_library(login, "TDS-Library");
 		tds_set_server(login, hostname);
 		tds_set_port(login, port);
-		tds_set_charset(login, charset);
+		tds_set_client_charset(login, charset);
 		tds_set_language(login, "us_english");
-		tds_set_packet(login, 512);
 		tds_set_passwd(login, password);
 	}
 
@@ -381,14 +433,15 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 }
 
 int
-tsql_handle_message(TDSCONTEXT * context, TDSSOCKET * tds, TDSMSGINFO * msg)
+tsql_handle_message(TDSCONTEXT * context, TDSSOCKET * tds, TDSMESSAGE * msg)
 {
 	if (msg->msg_number == 0) {
 		fprintf(stderr, "%s\n", msg->message);
 		return 0;
 	}
 
-	if (msg->msg_number != 5701 && msg->msg_number != 20018) {
+	if (msg->msg_number != 5701 && msg->msg_number != 5703
+	    && msg->msg_number != 20018) {
 		fprintf(stderr, "Msg %d, Level %d, State %d, Server %s, Line %d\n%s\n",
 			msg->msg_number, msg->msg_level, msg->msg_state, msg->server, msg->line_number, msg->message);
 	}
@@ -396,10 +449,37 @@ tsql_handle_message(TDSCONTEXT * context, TDSSOCKET * tds, TDSMSGINFO * msg)
 	return 0;
 }
 
+void
+slurp_input_file(char *fname, char **mybuf, int *bufsz, int *line)
+{
+	FILE *fp = NULL;
+	register char *n;
+	char linebuf[1024];
+	char *s = NULL;
+
+	if ((fp = fopen(fname, "r")) == NULL) {
+		fprintf(stderr, "Unable to open input file '%s': %s\n", fname, strerror(errno));
+		return;
+	}
+	while ((s = fgets(linebuf, sizeof(linebuf), fp)) != NULL) {
+		while (strlen(*mybuf) + strlen(s) + 2 > *bufsz) {
+			*bufsz *= 2;
+			*mybuf = (char *) realloc(*mybuf, *bufsz);
+		}
+		strcat(*mybuf, s);
+		n = strrchr(s, '\n');
+		if (n != NULL)
+			*n = '\0';
+		add_history(s);
+		(*line)++;
+	}
+}
+
+
 int
 main(int argc, char **argv)
 {
-	char *s = NULL;
+	char *s = NULL, *s2 = NULL, *cmd = NULL;
 	char prompt[20];
 	int line = 0;
 	char *mybuf;
@@ -407,7 +487,7 @@ main(int argc, char **argv)
 	TDSSOCKET *tds;
 	TDSLOGIN *login;
 	TDSCONTEXT *context;
-	TDSCONNECTINFO *connect_info;
+	TDSCONNECTION *connection;
 	int opt_flags = 0;
 
 	/* grab a login structure */
@@ -428,13 +508,13 @@ main(int argc, char **argv)
 	/* Try to open a connection */
 	tds = tds_alloc_socket(context, 512);
 	tds_set_parent(tds, NULL);
-	connect_info = tds_read_config_info(NULL, login, context->locale);
-	if (!connect_info || tds_connect(tds, connect_info) == TDS_FAIL) {
-		tds_free_connect(connect_info);
+	connection = tds_read_config_info(NULL, login, context->locale);
+	if (!connection || tds_connect(tds, connection) == TDS_FAIL) {
+		tds_free_connection(connection);
 		fprintf(stderr, "There was a problem connecting to the server\n");
 		exit(1);
 	}
-	tds_free_connect(connect_info);
+	tds_free_connection(connection);
 	/* give the buffer an initial size */
 	bufsz = 4096;
 	mybuf = (char *) malloc(bufsz);
@@ -445,24 +525,36 @@ main(int argc, char **argv)
 		if (s)
 			free(s);
 		s = readline(prompt);
-		if (!s || !strcmp(s, "exit") || !strcmp(s, "quit") || !strcmp(s, "bye")) {
+		if (s != NULL) {
+			if (s2)
+				free(s2);
+			s2 = strdup(s);	/* copy that can be safely mangled by str functions */
+			cmd = strtok(s2, " \t");
+		}
+		if (!cmd)
+			continue;
+
+		if (!s || !strcmp(cmd, "exit") || !strcmp(cmd, "quit") || !strcmp(cmd, "bye")) {
 			break;
 		}
-		if (!strcmp(s, "version")) {
+		if (!strcmp(cmd, "version")) {
 			tds_version(tds, mybuf);
 			printf("using TDS version %s\n", mybuf);
 			line = 0;
 			mybuf[0] = '\0';
 			continue;
 		}
-		if (!strncmp(s, "go", 2)) {
+		if (!strncmp(cmd, "go", 2)) {
 			line = 0;
-			get_opt_flags(s, &opt_flags);
+			get_opt_flags(s + 2, &opt_flags);
+			opt_flags ^= global_opt_flags;
 			do_query(tds, mybuf, opt_flags);
 			mybuf[0] = '\0';
-		} else if (!strcmp(s, "reset")) {
+		} else if (!strcmp(cmd, "reset")) {
 			line = 0;
 			mybuf[0] = '\0';
+		} else if (!strcmp(cmd, ":r")) {
+			slurp_input_file(strtok(NULL, " \t"), &mybuf, &bufsz, &line);
 		} else {
 			while (strlen(mybuf) + strlen(s) + 2 > bufsz) {
 				bufsz *= 2;
