@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -43,13 +43,52 @@
 #endif /* HAVE_STRING_H */
 
 #include "cspublic.h"
-#include "tdsconvert.h"
 #include "ctlib.h"
+#include "tdsconvert.h"
 #include "replacements.h"
 
-static char software_version[] = "$Id: cs.c,v 1.33 2003/01/04 13:06:57 freddy77 Exp $";
+static char software_version[] = "$Id: cs.c,v 1.53 2004/10/14 08:16:43 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
+static int _cs_datatype_length(int dtype);
+static CS_INT cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message);
+static CS_INT cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type);
+static CS_INT cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message);
+static CS_INT cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count);
+
+/* 	returns the fixed length of the specified data type, or 0 if not a 
+	fixed length data type */
+static int 
+_cs_datatype_length(int dtype)
+{
+	switch (dtype) {
+		case SYBINT1:
+			return 1;
+		case SYBINT2:
+			return 2;
+		case SYBINT4:
+			return 4;
+		case SYBINT8:
+			return 8;
+		case SYBFLT8:
+			return 8;
+		case SYBREAL:
+			return 4;
+		case SYBBIT:
+			return 1;
+		case SYBMONEY:
+			return 8;
+		case SYBMONEY4:
+			return 4;
+		case SYBDATETIME:
+			return 8;
+		case SYBDATETIME4:
+			return 4;
+		default:
+			return 0;
+	}
+}
+
 static const char *
 _cs_get_layer(int layer)
 {
@@ -188,6 +227,9 @@ CS_RETCODE
 cs_ctx_drop(CS_CONTEXT * ctx)
 {
 	if (ctx) {
+		_ct_diag_clearmsg(ctx, CS_ALLMSG_TYPE);
+		if (ctx->userdata)
+			free(ctx->userdata);
 		if (ctx->tds_ctx)
 			tds_free_context(ctx->tds_ctx);
 		free(ctx);
@@ -198,6 +240,9 @@ cs_ctx_drop(CS_CONTEXT * ctx)
 CS_RETCODE
 cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen)
 {
+/* declared  for - CS_USERDATA changes - swapna*/
+CS_INT maxcp;
+
 	if (action == CS_GET) {
 		if (buffer == NULL) {
 			return CS_SUCCEED;
@@ -205,10 +250,19 @@ cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS
 		switch (property) {
 		case CS_MESSAGE_CB:
 			*(void **) buffer = (void*) ctx->_cslibmsg_cb;
+			return CS_SUCCEED;
+		case CS_USERDATA:
+			maxcp = ctx->userdata_len;
+			if (outlen)
+				*outlen = maxcp;
+			if (maxcp > buflen)
+				maxcp = buflen;
+
+			memcpy(buffer, ctx->userdata, maxcp); 
+			
 			return CS_SUCCEED;
 		case CS_EXTRA_INF:
 		case CS_LOC_PROP:
-		case CS_USERDATA:
 		case CS_VERSION:
 			return CS_FAIL;
 			break;
@@ -217,11 +271,29 @@ cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS
 	/* CS_SET */
 	switch (property) {
 	case CS_MESSAGE_CB:
+		if ( ctx->cs_errhandletype == _CS_ERRHAND_INLINE) {
+			cs_diag_clearmsg(ctx, CS_UNUSED);
+		}
 		ctx->_cslibmsg_cb = (CS_CSLIBMSG_FUNC) buffer;
+		ctx->cs_errhandletype = _CS_ERRHAND_CB;
 		return CS_SUCCEED;
+	case CS_USERDATA:
+		if (ctx->userdata)
+			free(ctx->userdata);
+
+		if (buflen == CS_NULLTERM) {
+			maxcp = strlen(buffer) + 1;
+		} else {
+			maxcp = buflen;
+		}
+		ctx->userdata = (void *) malloc(maxcp);
+		ctx->userdata_len = maxcp;
+
+		memcpy(ctx->userdata, buffer, maxcp);
+		return CS_SUCCEED;
+
 	case CS_EXTRA_INF:
 	case CS_LOC_PROP:
-	case CS_USERDATA:
 	case CS_VERSION:
 		return CS_FAIL;
 		break;
@@ -241,18 +313,19 @@ unsigned char *dest;
 
 CS_RETCODE ret;
 
-	tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert()\n");
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert()\n");
 
 	src_type = _ct_get_server_type(srcfmt->datatype);
 	src_len = srcfmt ? srcfmt->maxlength : 0;
 	desttype = _ct_get_server_type(destfmt->datatype);
 	destlen = destfmt ? destfmt->maxlength : 0;
 
-	tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() srctype = %d (%d) desttype = %d (%d)\n",
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert() srctype = %d (%d) desttype = %d (%d)\n",
 		    src_type, src_len, desttype, destlen);
 
-	if (destlen <= 0)
-		return CS_SUCCEED;
+	if (!is_fixed_type(desttype) && (destlen <= 0)) {
+		return CS_FAIL;
+	}
 
 	dest = (unsigned char *) destdata;
 
@@ -260,7 +333,7 @@ CS_RETCODE ret;
 
 	if (srcdata == NULL) {
 
-		tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() srcdata is null\n");
+		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srcdata is null\n");
 		memset(dest, '\0', destlen);
 		if (resultlen != (CS_INT *) NULL)
 			*resultlen = 0;
@@ -272,10 +345,12 @@ CS_RETCODE ret;
 
 	if (src_type == desttype) {
 
-		tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() srctype = desttype\n");
+		tdsdump_log(TDS_DBG_FUNC, "cs_convert() srctype = desttype\n");
 		switch (desttype) {
 
+		case SYBLONGBINARY:
 		case SYBBINARY:
+		case SYBVARBINARY:
 		case SYBIMAGE:
 			if (src_len > destlen) {
 				ret = CS_FAIL;
@@ -302,11 +377,13 @@ CS_RETCODE ret;
 				}
 			}
 			break;
+
 		case SYBCHAR:
 		case SYBVARCHAR:
 		case SYBTEXT:
-			tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() desttype = character\n");
+			tdsdump_log(TDS_DBG_FUNC, "cs_convert() desttype = character\n");
 			if (src_len > destlen) {
+				tdsdump_log(TDS_DBG_FUNC, "src_len > destlen\n");
 				ret = CS_FAIL;
 			} else {
 				switch (destfmt->format) {
@@ -347,6 +424,7 @@ CS_RETCODE ret;
 					ret = CS_SUCCEED;
 					break;
 				default:
+					tdsdump_log(TDS_DBG_FUNC, "no destination format specified!\n");
 					ret = CS_FAIL;
 					break;
 				}
@@ -355,24 +433,21 @@ CS_RETCODE ret;
 		case SYBINT1:
 		case SYBINT2:
 		case SYBINT4:
+		case SYBINT8:
 		case SYBFLT8:
 		case SYBREAL:
 		case SYBBIT:
-		case SYBBITN:
 		case SYBMONEY:
 		case SYBMONEY4:
 		case SYBDATETIME:
 		case SYBDATETIME4:
-			if (src_len > destlen) {
-				ret = CS_FAIL;
-			} else {
-				memcpy(dest, srcdata, src_len);
-				if (resultlen != (CS_INT *) NULL)
-					*resultlen = src_len;
-				ret = CS_SUCCEED;
-			}
+			memcpy(dest, srcdata, _cs_datatype_length(src_type));
+			if (resultlen != (CS_INT *) NULL)
+				*resultlen = _cs_datatype_length(src_type);
+			ret = CS_SUCCEED;
 			break;
 
+		case SYBBITN:
 		case SYBNUMERIC:
 		case SYBDECIMAL:
 			if (src_len > destlen) {
@@ -395,7 +470,7 @@ CS_RETCODE ret;
 	}
 
 
-	/* src type == dest type */
+	/* src type != dest type */
 	/* set the output precision/scale for conversions to numeric type */
 	if (is_numeric_type(desttype)) {
 		cres.n.precision = destfmt->precision;
@@ -406,10 +481,10 @@ CS_RETCODE ret;
 			cres.n.scale = srcfmt->scale;
 	}
 
-	tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() calling tds_convert\n");
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert() calling tds_convert\n");
 	len = tds_convert(ctx->tds_ctx, src_type, (TDS_CHAR*) srcdata, src_len, desttype, &cres);
 
-	tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() tds_convert returned %d\n", len);
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert() tds_convert returned %d\n", len);
 
 	switch (len) {
 	case TDS_CONVERT_NOAVAIL:
@@ -440,6 +515,7 @@ CS_RETCODE ret;
 
 	switch (desttype) {
 	case SYBBINARY:
+	case SYBVARBINARY:
 	case SYBIMAGE:
 
 		if (len > destlen) {
@@ -477,6 +553,12 @@ CS_RETCODE ret;
 			*resultlen = 4;
 		ret = CS_SUCCEED;
 		break;
+	case SYBINT8:
+		memcpy(dest, &(cres.bi), 8);
+		if (resultlen != (CS_INT *) NULL)
+			*resultlen = 8;
+		ret = CS_SUCCEED;
+		break;
 	case SYBFLT8:
 		memcpy(dest, &(cres.f), 8);
 		if (resultlen != (CS_INT *) NULL)
@@ -491,7 +573,7 @@ CS_RETCODE ret;
 		break;
 	case SYBMONEY:
 
-		tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() copying %d bytes to src\n", sizeof(TDS_MONEY));
+		tdsdump_log(TDS_DBG_FUNC, "cs_convert() copying %d bytes to src\n", (int) sizeof(TDS_MONEY));
 		memcpy(dest, &(cres.m), sizeof(TDS_MONEY));
 		if (resultlen != (CS_INT *) NULL)
 			*resultlen = sizeof(TDS_MONEY);
@@ -532,9 +614,9 @@ CS_RETCODE ret;
 			switch (destfmt->format) {
 
 			case CS_FMT_NULLTERM:
-				tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() FMT_NULLTERM\n");
+				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_NULLTERM\n");
 				if (len == destlen) {
-					tdsdump_log(TDS_DBG_FUNC, "%L not enough room for data + a null terminator - error\n");
+					tdsdump_log(TDS_DBG_FUNC, "not enough room for data + a null terminator - error\n");
 					ret = CS_FAIL;	/* not enough room for data + a null terminator - error */
 				} else {
 					memcpy(dest, cres.c, len);
@@ -546,7 +628,7 @@ CS_RETCODE ret;
 				break;
 
 			case CS_FMT_PADBLANK:
-				tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() FMT_PADBLANK\n");
+				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADBLANK\n");
 				/* strcpy here can lead to a small buffer overflow */
 				memcpy(dest, cres.c, len);
 				for (i = len; i < destlen; i++)
@@ -557,7 +639,7 @@ CS_RETCODE ret;
 				break;
 
 			case CS_FMT_PADNULL:
-				tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() FMT_PADNULL\n");
+				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_PADNULL\n");
 				/* strcpy here can lead to a small buffer overflow */
 				memcpy(dest, cres.c, len);
 				for (i = len; i < destlen; i++)
@@ -567,7 +649,7 @@ CS_RETCODE ret;
 				ret = CS_SUCCEED;
 				break;
 			case CS_FMT_UNUSED:
-				tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() FMT_UNUSED\n");
+				tdsdump_log(TDS_DBG_FUNC, "cs_convert() FMT_UNUSED\n");
 				memcpy(dest, cres.c, len);
 				if (resultlen != (CS_INT *) NULL)
 					*resultlen = len;
@@ -584,15 +666,15 @@ CS_RETCODE ret;
 		ret = CS_FAIL;
 		break;
 	}
-	tdsdump_log(TDS_DBG_FUNC, "%L inside cs_convert() returning  %d\n", ret);
+	tdsdump_log(TDS_DBG_FUNC, "cs_convert() returning  %d\n", ret);
 	return (ret);
 }
 
 CS_RETCODE
 cs_dt_crack(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
 {
-CS_DATETIME *dt;
-CS_DATETIME4 *dt4;
+TDS_DATETIME *dt;
+TDS_DATETIME4 *dt4;
 time_t tmp_secs_from_epoch;
 struct tm *t;
 TDSDATEREC dr;
@@ -615,6 +697,7 @@ TDSDATEREC dr;
 	daterec->datehour = dr.hour;
 	daterec->dateminute = dr.minute;
 	daterec->datesecond = dr.second;
+	daterec->datemsecond = dr.millisecond;
 	daterec->datetzone = 0;
 
 	return CS_SUCCEED;
@@ -668,7 +751,7 @@ CS_RETCODE
 cs_calc(CS_CONTEXT * ctx, CS_INT op, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_VOID * dest)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_calc()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_calc()\n");
 	return CS_FAIL;
 }
 
@@ -676,7 +759,7 @@ CS_RETCODE
 cs_cmp(CS_CONTEXT * ctx, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_INT * result)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_cmp()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_cmp()\n");
 	return CS_FAIL;
 }
 
@@ -684,16 +767,64 @@ CS_RETCODE
 cs_conv_mult(CS_CONTEXT * ctx, CS_LOCALE * srcloc, CS_LOCALE * destloc, CS_INT * conv_multiplier)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_conv_mult()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_conv_mult()\n");
 	return CS_FAIL;
 }
 
 CS_RETCODE
 cs_diag(CS_CONTEXT * ctx, CS_INT operation, CS_INT type, CS_INT idx, CS_VOID * buffer)
 {
+	switch (operation) {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_diag()\n");
-	return CS_FAIL;
+		case CS_INIT:
+			if ( ctx->cs_errhandletype == _CS_ERRHAND_CB) {
+				/* contrary to the manual page you don't seem to */
+				/* be able to turn on inline message handling    */
+				/* using cs_diag, once a callback is installed!  */
+				return CS_FAIL;
+			}
+			ctx->cs_errhandletype = _CS_ERRHAND_INLINE;
+			ctx->cs_diag_msglimit = CS_NO_LIMIT;
+			ctx->_cslibmsg_cb = (CS_CSLIBMSG_FUNC) cs_diag_storemsg; 
+			break;
+		case CS_MSGLIMIT:
+			if ( ctx->cs_errhandletype != _CS_ERRHAND_INLINE) {
+				return CS_FAIL;
+			}
+			ctx->cs_diag_msglimit = *(CS_INT *)buffer;
+			break;
+		case CS_CLEAR:
+			if ( ctx->cs_errhandletype != _CS_ERRHAND_INLINE) {
+				return CS_FAIL;
+			}
+			return (cs_diag_clearmsg(ctx, type));
+
+			break;
+		case CS_GET:
+			if ( ctx->cs_errhandletype != _CS_ERRHAND_INLINE) {
+				return CS_FAIL;
+			}
+			if (buffer == NULL)
+				return CS_FAIL;
+
+			if (idx == 0 || (ctx->cs_diag_msglimit != CS_NO_LIMIT && idx > ctx->cs_diag_msglimit) )
+				return CS_FAIL;
+
+			return (cs_diag_getmsg(ctx, idx, (CS_CLIENTMSG *)buffer)); 
+			
+			break;
+		case CS_STATUS:
+			if ( ctx->cs_errhandletype != _CS_ERRHAND_INLINE) {
+				return CS_FAIL;
+			}
+			if (buffer == NULL) 
+				return CS_FAIL;
+
+			return (cs_diag_countmsg(ctx, (CS_INT *)buffer));
+			break;
+	}
+	return CS_SUCCEED;
+		
 }
 
 CS_RETCODE
@@ -701,7 +832,7 @@ cs_manage_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_CHAR * src
 		  CS_CHAR * destname, CS_INT destnamelen, CS_INT * conv_multiplier, CS_CONV_FUNC * func)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_manage_convert()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_manage_convert()\n");
 	return CS_FAIL;
 }
 
@@ -709,7 +840,7 @@ CS_RETCODE
 cs_objects(CS_CONTEXT * ctx, CS_INT action, CS_OBJNAME * objname, CS_OBJDATA * objdata)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_objects()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_objects()\n");
 	return CS_FAIL;
 }
 
@@ -717,7 +848,7 @@ CS_RETCODE
 cs_set_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_INT desttype, CS_CONV_FUNC * func)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_set_convert()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_set_convert()\n");
 	return CS_FAIL;
 }
 
@@ -725,7 +856,7 @@ CS_RETCODE
 cs_setnull(CS_CONTEXT * ctx, CS_DATAFMT * datafmt, CS_VOID * buffer, CS_INT buflen)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_setnull()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_setnull()\n");
 	return CS_FAIL;
 }
 
@@ -734,7 +865,7 @@ cs_strcmp(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_INT type, CS_CHAR * str1, CS_
 	  CS_INT * result)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_strcmp()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_strcmp()\n");
 	return CS_FAIL;
 }
 
@@ -742,7 +873,7 @@ CS_RETCODE
 cs_time(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen, CS_DATEREC * daterec)
 {
 
-	tdsdump_log(TDS_DBG_FUNC, "%L UNIMPLEMENTED cs_time()\n");
+	tdsdump_log(TDS_DBG_FUNC, "UNIMPLEMENTED cs_time()\n");
 	return CS_FAIL;
 }
 
@@ -753,3 +884,107 @@ cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * res
 	*result = (tds_willconvert(srctype, desttype) ? CS_TRUE : CS_FALSE);
 	return CS_SUCCEED;
 }
+
+static CS_INT 
+cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message)
+{
+
+struct cs_diag_msg **curptr;
+CS_INT msg_count = 0;
+
+	curptr = &(context->msgstore);
+
+	/* if we already have a list of messages, */
+	/* go to the end of the list...           */
+
+	while (*curptr != (struct cs_diag_msg *)NULL) {
+		msg_count++;
+		curptr = &((*curptr)->next);
+	}
+
+	/* messages over and above the agreed limit */
+	/* are simply discarded...                  */
+
+	if (context->cs_diag_msglimit != CS_NO_LIMIT &&
+		msg_count >= context->cs_diag_msglimit) {
+		return CS_FAIL;
+	}
+
+	*curptr = (struct cs_diag_msg *) malloc(sizeof(struct cs_diag_msg));
+	if (*curptr == (struct cs_diag_msg *)NULL) { 
+		return CS_FAIL;
+	} else {
+		(*curptr)->next = (struct cs_diag_msg *)NULL;
+		(*curptr)->msg  = malloc(sizeof(CS_CLIENTMSG));
+		if ((*curptr)->msg == (CS_CLIENTMSG *) NULL) {
+			return CS_FAIL;
+		} else {
+			memcpy((*curptr)->msg, message, sizeof(CS_CLIENTMSG));
+		}
+	}
+
+	return CS_SUCCEED;
+}
+
+static CS_INT 
+cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message)
+{
+
+struct cs_diag_msg *curptr;
+CS_INT msg_count = 0, msg_found = 0;
+
+	curptr = context->msgstore;
+
+	/* if we already have a list of messages, */
+	/* go to the end of the list...           */
+
+	while (curptr != (struct cs_diag_msg *)NULL) {
+		msg_count++;
+		if (msg_count == idx) {
+			msg_found++;
+			break;
+		}
+		curptr = curptr->next;
+	}
+	if (msg_found) {
+		memcpy(message, curptr->msg, sizeof(CS_CLIENTMSG));
+		return CS_SUCCEED;
+	} else {
+		return CS_NOMSG;
+	}
+}
+
+static CS_INT 
+cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type)
+{
+
+struct cs_diag_msg *curptr, *freeptr;
+
+	curptr = context->msgstore;
+	context->msgstore = NULL;
+
+	while (curptr != (struct cs_diag_msg *)NULL ) {
+        	freeptr = curptr;
+		curptr = freeptr->next;
+        	if (freeptr->msg)
+        		free(freeptr->msg);
+        	free(freeptr);
+	}
+	return CS_SUCCEED;
+}
+
+static CS_INT 
+cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count)
+{
+struct cs_diag_msg *curptr;
+CS_INT msg_count = 0;
+
+	curptr = context->msgstore;
+
+	while (curptr != (struct cs_diag_msg *)NULL) {
+		msg_count++;
+		curptr = curptr->next;
+	}
+	*count = msg_count;
+	return CS_SUCCEED;
+}
