@@ -4,14 +4,19 @@
 
 #include <stdio.h>
 
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
+
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
 #include <ctpublic.h>
 #include "common.h"
+#include "ctlib.h"
 
-static char software_version[] = "$Id: common.c,v 1.10 2003/01/26 18:42:54 freddy77 Exp $";
+static char software_version[] = "$Id: common.c,v 1.14 2004/07/21 19:28:01 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 char USER[512];
@@ -19,6 +24,8 @@ char SERVER[512];
 char PASSWORD[512];
 char DATABASE[512];
 
+COMMON_PWD common_pwd = {0};
+
 int cslibmsg_cb_invoked = 0;
 int clientmsg_cb_invoked = 0;
 int servermsg_cb_invoked = 0;
@@ -29,6 +36,14 @@ read_login_info(void)
 	FILE *in;
 	char line[512];
 	char *s1, *s2;
+	
+	if (common_pwd.initialized) {
+		strcpy(USER, common_pwd.USER);
+		strcpy(SERVER, common_pwd.SERVER);
+		strcpy(PASSWORD, common_pwd.PASSWORD);
+		strcpy(DATABASE, common_pwd.DATABASE);
+		return CS_SUCCEED;
+	}
 
 	in = fopen("../../../PWD", "r");
 	if (!in) {
@@ -62,6 +77,7 @@ try_ctlogin(CS_CONTEXT ** ctx, CS_CONNECTION ** conn, CS_COMMAND ** cmd, int ver
 {
 CS_RETCODE ret;
 char query[30];
+TDSCONTEXT *tds_ctx;
 
 	/* read login information from PWD file */
 	ret = read_login_info();
@@ -78,6 +94,14 @@ char query[30];
 		}
 		return ret;
 	}
+
+	/* Force default date format, some tests rely on it */
+	tds_ctx = (TDSCONTEXT *) (*ctx)->tds_ctx;
+	if (tds_ctx && tds_ctx->locale && tds_ctx->locale->date_fmt) {
+		free(tds_ctx->locale->date_fmt);
+		tds_ctx->locale->date_fmt = strdup("%b %e %Y %I:%M%p");
+	}
+
 	ret = ct_init(*ctx, CS_VERSION_100);
 	if (ret != CS_SUCCEED) {
 		if (verbose) {
@@ -205,7 +229,7 @@ CS_INT result_type;
 }
 
 CS_INT
-cslibmsg_cb(CS_CONTEXT * context, CS_CLIENTMSG * errmsg)
+cslibmsg_cb(CS_CONTEXT * connection, CS_CLIENTMSG * errmsg)
 {
 	cslibmsg_cb_invoked++;
 	fprintf(stderr, "\nCS-Library Message:\n");
@@ -237,7 +261,7 @@ clientmsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_CLIENTMSG * er
 }
 
 CS_RETCODE
-servermsg_cb(CS_CONNECTION * connection, CS_COMMAND * cmd, CS_SERVERMSG * srvmsg)
+servermsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_SERVERMSG * srvmsg)
 {
 	servermsg_cb_invoked++;
 	fprintf(stderr, "\nServer Message:\n");
