@@ -20,7 +20,7 @@
 
 
 
-static char software_version[] = "$Id: t0002.c,v 1.7 2002/11/20 13:47:06 freddy77 Exp $";
+static char software_version[] = "$Id: t0002.c,v 1.9 2004/09/09 08:54:49 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -62,7 +62,7 @@ main(int argc, char **argv)
 
 	set_malloc_options();
 
-	read_login_info();
+	read_login_info(argc, argv);
 
 	fprintf(stdout, "Start\n");
 	add_bread_crumb();
@@ -89,6 +89,8 @@ main(int argc, char **argv)
 	if (strlen(DATABASE))
 		dbuse(dbproc, DATABASE);
 	add_bread_crumb();
+	dbloginfree(login);
+	add_bread_crumb();
 
 	dbsetopt(dbproc, DBBUFFER, "10", 0);
 	add_bread_crumb();
@@ -202,5 +204,6 @@ main(int argc, char **argv)
 
 
 	fprintf(stdout, "dblib %s on %s\n", (failed ? "failed!" : "okay"), __FILE__);
+	free_bread_crumb();
 	return failed ? 1 : 0;
 }
