@@ -27,7 +27,7 @@
 
 
 
-static char software_version[] = "$Id: t0010.c,v 1.8 2002/11/20 13:47:06 freddy77 Exp $";
+static char software_version[] = "$Id: t0010.c,v 1.10 2004/09/09 08:54:49 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -43,7 +43,7 @@ main(int argc, char **argv)
 
 	set_malloc_options();
 
-	read_login_info();
+	read_login_info(argc, argv);
 
 	fprintf(stdout, "Start\n");
 	add_bread_crumb();
@@ -71,6 +71,8 @@ main(int argc, char **argv)
 	if (strlen(DATABASE))
 		dbuse(dbproc, DATABASE);
 	add_bread_crumb();
+	dbloginfree(login);
+	add_bread_crumb();
 
 #ifdef MICROSOFT_DBLIB
 	dbsetopt(dbproc, DBBUFFER, "100");
@@ -158,6 +160,11 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	add_bread_crumb();
+	dbexit();
+	add_bread_crumb();
+
 	fprintf(stderr, "dblib passed for %s\n", __FILE__);
+	free_bread_crumb();
 	return 0;
 }
