@@ -17,7 +17,7 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0012.c,v 1.14 2003/01/26 18:42:54 freddy77 Exp $";
+static char software_version[] = "$Id: t0012.c,v 1.16.2.1 2004/11/29 08:40:01 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 int failed = 0;
 
@@ -35,7 +35,7 @@ main(int argc, char *argv[])
 
 	set_malloc_options();
 
-	read_login_info();
+	read_login_info(argc, argv);
 	fprintf(stdout, "Start\n");
 	dbinit();
 
@@ -55,6 +55,7 @@ main(int argc, char *argv[])
 	if (strlen(DATABASE)) {
 		dbuse(dbproc, DATABASE);
 	}
+	dbloginfree(login);
 	fprintf(stdout, "After logon\n");
 
 	fprintf(stdout, "Dropping table\n");
@@ -106,7 +107,7 @@ main(int argc, char *argv[])
 		dbdatecrack(dbproc, &dateinfo, &mydatetime);
 
 		/* Print the parts of the creation date */
-#if MSDBLIB
+#ifdef MSDBLIB
 		printf("\tYear = %d.\n", dateinfo.year);
 		printf("\tMonth = %d.\n", dateinfo.month);
 		printf("\tDay of month = %d.\n", dateinfo.day);
@@ -133,5 +134,6 @@ main(int argc, char *argv[])
 	dbexit();
 
 	fprintf(stdout, "dblib %s on %s\n", (failed ? "failed!" : "okay"), __FILE__);
+	free_bread_crumb();
 	return failed ? 1 : 0;
 }
