@@ -17,7 +17,7 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: t0016.c,v 1.16 2002/11/23 17:11:00 freddy77 Exp $";
+static char software_version[] = "$Id: t0016.c,v 1.20 2004/09/09 08:54:49 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 int failed = 0;
 
@@ -38,7 +38,7 @@ main(int argc, char *argv[])
 
 	set_malloc_options();
 
-	read_login_info();
+	read_login_info(argc, argv);
 	fprintf(stdout, "Start\n");
 	dbinit();
 
@@ -57,6 +57,7 @@ main(int argc, char *argv[])
 	if (strlen(DATABASE)) {
 		dbuse(dbproc, DATABASE);
 	}
+	dbloginfree(login);
 	fprintf(stdout, "After logon\n");
 
 	fprintf(stdout, "Dropping table\n");
