@@ -20,12 +20,12 @@
 #ifndef CONVERT_TDS2SQL_h
 #define CONVERT_TDS2SQL_h
 
-static char rcsid_convert_tds2sql_h[] = "$Id: convert_tds2sql.h,v 1.3 2002/11/08 16:08:43 freddy77 Exp $";
-static void *no_unused_convert_tds2sql_h_warn[] = { rcsid_convert_tds2sql_h, no_unused_convert_tds2sql_h_warn };
+static const char rcsid_convert_tds2sql_h[] = "$Id: convert_tds2sql.h,v 1.4 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_convert_tds2sql_h_warn[] = { rcsid_convert_tds2sql_h, no_unused_convert_tds2sql_h_warn };
 
 
 
 TDS_INT convert_tds2sql(TDSCONTEXT * context, int srctype, TDS_CHAR * src, TDS_UINT srclen, int desttype, TDS_CHAR * dest,
-			TDS_UINT destlen);
+			SQLULEN destlen);
 
 #endif
