@@ -24,21 +24,23 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-#ifdef UNIXODBC
+#if defined(UNIXODBC) || defined(TDS_NO_DM)
 #include <sql.h>
 #include <sqlext.h>
-#else
+#else /* iODBC */
 #include "isql.h"
 #include "isqlext.h"
 #endif
 
 
-static char rcsid_prepare_query_h[] = "$Id: prepare_query.h,v 1.4 2002/11/08 16:08:43 freddy77 Exp $";
-static void *no_unused_prepare_query_h_warn[] = { rcsid_prepare_query_h, no_unused_prepare_query_h_warn };
+static const char rcsid_prepare_query_h[] = "$Id: prepare_query.h,v 1.10 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_prepare_query_h_warn[] = { rcsid_prepare_query_h, no_unused_prepare_query_h_warn };
 
 
-int prepare_call(struct _hstmt *stmt);
-int start_parse_prepared_query(struct _hstmt *stmt);
-int continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLINTEGER StrLen_or_Ind);
+SQLRETURN prepare_call(struct _hstmt *stmt);
+SQLRETURN native_sql(struct _hdbc *dbc, char *s);
+int parse_prepared_query(struct _hstmt *stmt, int start, int compute_row);
+int start_parse_prepared_query(struct _hstmt *stmt, int compute_row);
+int continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLLEN StrLen_or_Ind);
 
 #endif
