@@ -20,9 +20,10 @@
 #ifndef SQL2TDS_H
 #define SQL2TDS_H
 
-static char rcsid_sql2tds_h[] = "$Id: sql2tds.h,v 1.1 2002/11/30 14:11:36 freddy77 Exp $";
-static void *no_unused_sql2tds_h_warn[] = { rcsid_sql2tds_h, no_unused_sql2tds_h_warn };
+static const char rcsid_sql2tds_h[] = "$Id: sql2tds.h,v 1.8 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_sql2tds_h_warn[] = { rcsid_sql2tds_h, no_unused_sql2tds_h_warn };
 
-int sql2tds(TDSCONTEXT * context, struct _sql_param_info *param, TDSPARAMINFO * info, TDSCOLINFO * curcol);
+SQLRETURN sql2tds(TDS_DBC * dbc, const struct _drecord *drec_ipd, const struct _drecord *drec_apd, TDSPARAMINFO * info, int nparam,
+		  int compute_row);
 
 #endif /* SQL2TDS_H */
