@@ -1,17 +1,7 @@
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STDLIB_H
-#include <stdlib.h>
-#endif /* HAVE_STDLIB_H */
-
 #include "common.h"
 
 
-static char software_version[] = "$Id: connect.c,v 1.4 2003/01/26 18:42:54 freddy77 Exp $";
+static char software_version[] = "$Id: connect.c,v 1.8 2003/12/20 13:23:38 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 static void init_connect(void);
@@ -36,6 +26,29 @@ main(int argc, char *argv[])
 	int res;
 	char tmp[2048];
 	SQLSMALLINT len;
+	int failures = 0;
+
+	if (read_login_info())
+		exit(1);
+
+	/*
+	 * prepare our odbcinst.ini 
+	 * is better to do it before connect cause uniODBC cache INIs
+	 * the name must be odbcinst.ini cause unixODBC accept only this name
+	 */
+	if (DRIVER[0]) {
+		FILE *f = fopen("odbcinst.ini", "w");
+
+		if (f) {
+			fprintf(f, "[FreeTDS]\nDriver = %s\n", DRIVER);
+			fclose(f);
+			/* force iODBC */
+			setenv("ODBCINSTINI", "./odbcinst.ini", 1);
+			setenv("SYSODBCINSTINI", "./odbcinst.ini", 1);
+			/* force unixODBC (only directory) */
+			setenv("ODBCSYSINI", ".", 1);
+		}
+	}
 
 	printf("SQLConnect connect..\n");
 	Connect();
@@ -45,8 +58,8 @@ main(int argc, char *argv[])
 	printf("connect string DSN connect..\n");
 	init_connect();
 	sprintf(tmp, "DSN=%s;UID=%s;PWD=%s;DATABASE=%s;", SERVER, USER, PASSWORD, DATABASE);
-	res = SQLDriverConnect(Connection, NULL, (SQLCHAR*) tmp, SQL_NTS, (SQLCHAR*) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT);
-	if (res != SQL_SUCCESS) {
+	res = SQLDriverConnect(Connection, NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT);
+	if (!SQL_SUCCEEDED(res)) {
 		printf("Unable to open data source (ret=%d)\n", res);
 		CheckReturn();
 		exit(1);
@@ -56,15 +69,32 @@ main(int argc, char *argv[])
 	/* try connect string using old SERVERNAME specification */
 	printf("connect string SERVERNAME connect..\n");
 	printf("odbcinst.ini must be configured with FreeTDS driver..\n");
+
+	/* this is expected to work with unixODBC */
 	init_connect();
 	sprintf(tmp, "DRIVER=FreeTDS;SERVERNAME=%s;UID=%s;PWD=%s;DATABASE=%s;", SERVER, USER, PASSWORD, DATABASE);
-	res = SQLDriverConnect(Connection, NULL, (SQLCHAR*) tmp, SQL_NTS, (SQLCHAR*) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT);
-	if (res != SQL_SUCCESS) {
+	res = SQLDriverConnect(Connection, NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT);
+	if (!SQL_SUCCEEDED(res)) {
 		printf("Unable to open data source (ret=%d)\n", res);
+		++failures;
+	}
+	Disconnect();
+
+	/* this is expected to work with iODBC */
+	init_connect();
+	sprintf(tmp, "DRIVER=%s;SERVERNAME=%s;UID=%s;PWD=%s;DATABASE=%s;", DRIVER, SERVER, USER, PASSWORD, DATABASE);
+	res = SQLDriverConnect(Connection, NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT);
+	if (!SQL_SUCCEEDED(res)) {
+		printf("Unable to open data source (ret=%d)\n", res);
+		++failures;
+	}
+	Disconnect();
+
+	/* at least one should success.. */
+	if (failures > 1) {
 		CheckReturn();
 		exit(1);
 	}
-	Disconnect();
 
 	printf("Done.\n");
 	return 0;
