@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +22,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
+#include <assert.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -43,11 +44,16 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
+#if HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif /* HAVE_ARPA_INET_H */
+
 #include "tds.h"
+#include "tdsiconv.h"
 #include "tdssrv.h"
 #include "tdsstring.h"
 
-static char software_version[] = "$Id: login.c,v 1.19.2.1 2003/02/26 04:19:10 jklowden Exp $";
+static char software_version[] = "$Id: login.c,v 1.40 2004/06/15 13:16:55 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 unsigned char *
@@ -68,11 +74,11 @@ tds7_decrypt_pass(const unsigned char *crypt_pass, int len, unsigned char *clear
 TDSSOCKET *
 tds_listen(int ip_port)
 {
-TDSCONTEXT *context;
-TDSSOCKET *tds;
-struct sockaddr_in sin;
-int fd, s;
-size_t len;
+	TDSCONTEXT *context;
+	TDSSOCKET *tds;
+	struct sockaddr_in sin;
+	TDS_SYS_SOCKET fd, s;
+	socklen_t len;
 
 	sin.sin_addr.s_addr = INADDR_ANY;
 	sin.sin_port = htons((short) ip_port);
@@ -99,12 +105,12 @@ size_t len;
 	return tds;
 }
 
-static char *tds_read_string(TDSSOCKET * tds, char **s, int size);
+static char *tds_read_string(TDSSOCKET * tds, DSTR * s, int size);
 
 void
 tds_read_login(TDSSOCKET * tds, TDSLOGIN * login)
 {
-char *blockstr;
+	DSTR blockstr;
 
 /*
 	while (len = tds_read_packet(tds)) {
@@ -131,11 +137,11 @@ char *blockstr;
 	tds_get_n(tds, NULL, 3);	/* magic */
 	tds_read_string(tds, &login->language, 30);
 	tds_get_n(tds, NULL, 14);	/* magic */
-	tds_read_string(tds, &login->char_set, 30);
+	tds_read_string(tds, &login->server_charset, 30);
 	tds_get_n(tds, NULL, 1);	/* magic */
 	tds_read_string(tds, &blockstr, 6);
-	printf("block size %s\n", blockstr);
-	login->block_size = atoi(blockstr);
+	printf("block size %s\n", tds_dstr_cstr(&blockstr));
+	login->block_size = atoi(tds_dstr_cstr(&blockstr));
 	tds_dstr_free(&blockstr);
 	tds_get_n(tds, NULL, tds->in_len - tds->in_pos);	/* read junk at end */
 }
@@ -146,7 +152,8 @@ tds7_read_string(TDSSOCKET * tds, int len)
 	char *s;
 
 	s = (char *) malloc(len + 1);
-	tds_get_string(tds, s, len);
+	/* FIXME possible truncation on char conversion ? */
+	len = tds_get_string(tds, len, s, len);
 	s[len] = 0;
 	return s;
 
@@ -155,11 +162,12 @@ tds7_read_string(TDSSOCKET * tds, int len)
 int
 tds7_read_login(TDSSOCKET * tds, TDSLOGIN * login)
 {
-int a;
-int host_name_len, user_name_len, password_len, app_name_len, server_name_len;
-int library_name_len, language_name_len;
-char *unicode_string;
-char *buf;
+	int a;
+	int host_name_len, user_name_len, app_name_len, server_name_len;
+	int library_name_len, language_name_len;
+	size_t unicode_len, password_len;
+	char *unicode_string;
+	char *buf, *pbuf;
 
 	a = tds_get_smallint(tds);	/*total packet size */
 	tds_get_n(tds, NULL, 5);
@@ -196,12 +204,21 @@ char *buf;
 	tds_dstr_set(&login->host_name, tds7_read_string(tds, host_name_len));
 	tds_dstr_set(&login->user_name, tds7_read_string(tds, user_name_len));
 
-	unicode_string = (char *) malloc(password_len * 2);
+	unicode_len = password_len * 2;
+	unicode_string = (char *) malloc(unicode_len);
 	buf = (char *) malloc(password_len + 1);
-	tds_get_n(tds, unicode_string, password_len * 2);
-	tds7_decrypt_pass((unsigned char*) unicode_string, password_len * 2, (unsigned char*)unicode_string);
-	tds7_unicode2ascii(tds, unicode_string, buf, password_len);
-	buf[password_len] = 0;
+	tds_get_n(tds, unicode_string, unicode_len);
+	tds7_decrypt_pass((unsigned char *) unicode_string, unicode_len, (unsigned char *) unicode_string);
+	pbuf = buf;
+	
+	memset(&tds->char_convs[client2ucs2]->suppress, 0, sizeof(tds->char_convs[client2ucs2]->suppress));
+	a = tds_iconv(tds, tds->char_convs[client2ucs2], to_client, (const char **) &unicode_string, &unicode_len, &pbuf,
+			 &password_len);
+	if (a < 0 ) {
+		fprintf(stderr, "error: %s:%d: tds7_read_login: tds_iconv() failed\n", __FILE__, __LINE__);
+		assert(-1 != a);
+	}
+	*pbuf = '\0';
 	tds_dstr_set(&login->password, buf);
 	free(unicode_string);
 
@@ -221,14 +238,14 @@ char *buf;
 	tds_get_n(tds, NULL, 7);
 	a = tds_get_byte(tds);	/*0x30 */
 	tds_get_n(tds, NULL, 3);
-	tds_dstr_copy(&login->char_set, "");	/*empty char_set for TDS 7.0 */
+	tds_dstr_copy(&login->server_charset, "");	/*empty char_set for TDS 7.0 */
 	login->block_size = 0;	/*0 block size for TDS 7.0 */
 	login->encrypted = 1;
 	return (0);
 
 }
 static char *
-tds_read_string(TDSSOCKET * tds, char **s, int size)
+tds_read_string(TDSSOCKET * tds, DSTR * s, int size)
 {
 	char *tempbuf;
 	int len;
@@ -243,3 +260,67 @@ tds_read_string(TDSSOCKET * tds, char **s, int size)
 	tds_dstr_set(s, tempbuf);
 	return tempbuf;
 }
+
+
+/**
+ * Allocate a TDSLOGIN structure, read a login packet into it, and return it.
+ * This is smart enough to distinguish between TDS4/5 or TDS7.  The calling
+ * function should call tds_free_login() on the returned structure when it is
+ * no longer needed.
+ * \param tds  The socket to read from
+ * \return  Returns NULL if no login was received.  The calling function can
+ * use IS_TDSDEAD(tds) to distinguish between an error/shutdown on the socket,
+ * or the receipt of an unexpected packet type.  In the latter case,
+ * tds->in_flag will indicate the return type.
+ */
+TDSLOGIN *
+tds_alloc_read_login(TDSSOCKET * tds)
+{
+	TDSLOGIN * login;
+
+	/*
+	 * This should only be done on a server connection, and the server
+	 * always sends 0x04 packets.
+	 */
+	tds->out_flag = 0x04;
+
+	/* Pre-read the next packet so we know what kind of packet it is */
+	if (tds_read_packet(tds) < 1) {
+		return NULL;
+	}
+
+	/* Allocate the login packet */
+	login = tds_alloc_login();
+
+	/* Use the packet type to determine which login format to expect */
+	switch (tds->in_flag) {
+	case 0x02: /* TDS4/5 login */
+		tds_read_login(tds, login);
+		if (login->block_size == 0) {
+			login->block_size = 512;
+		}
+		break;
+
+	case 0x10: /* TDS7+ login */
+		tds7_read_login(tds, login);
+		break;
+
+	case 0x12: /* TDS7+ prelogin, hopefully followed by a login */
+		tds7_read_login(tds, login);
+		tds_send_253_token(tds, TDS_DONE_FINAL, 0);
+		tds_flush_packet(tds);
+		if (tds_read_packet(tds) < 0 || tds->in_flag != 0x10) {
+			return NULL;
+		}
+		tds7_read_login(tds, login);
+		break;
+
+	default:
+		/* unexpected packet */
+		tds_free_login(login);
+		return NULL;
+	}
+
+	/* Return it */
+	return login;
+}
