@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,10 +25,12 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
+#include <assert.h>
+
 #include "tds.h"
 #include "tdssrv.h"
 
-static char software_version[] = "$Id: server.c,v 1.14 2002/12/25 11:19:32 freddy77 Exp $";
+static char software_version[] = "$Id: server.c,v 1.19 2004/05/27 14:50:06 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 void
@@ -36,10 +38,20 @@ tds_env_change(TDSSOCKET * tds, int type, const char *oldvalue, const char *newv
 {
 	TDS_SMALLINT totsize;
 
-	tds_put_byte(tds, TDS_ENVCHANGE_TOKEN);
+	/* If oldvalue is NULL, treat it like "" */
+	if (oldvalue == NULL)
+		oldvalue = "";
+
+	/*
+	 * NOTE: I don't know why each type of environment value has a different
+	 * format.  According to the TDS5 specifications, they should all use
+	 * the same format.   The code for the TDS_ENV_CHARSET case *should*
+	 * work for all environment values.  -- Steve Kirkendall
+	 */
+
 	switch (type) {
-		/* database */
-	case 1:
+	case TDS_ENV_DATABASE:
+		tds_put_byte(tds, TDS_ENVCHANGE_TOKEN);
 		/* totsize = type + newlen + newvalue + oldlen + oldvalue  */
 		totsize = strlen(oldvalue) + strlen(newvalue) + 3;
 		tds_put_smallint(tds, totsize);
@@ -49,8 +61,8 @@ tds_env_change(TDSSOCKET * tds, int type, const char *oldvalue, const char *newv
 		tds_put_byte(tds, strlen(oldvalue));
 		tds_put_n(tds, oldvalue, strlen(oldvalue));
 		break;
-		/* language */
-	case 2:
+	case TDS_ENV_LANG:
+		tds_put_byte(tds, TDS_ENVCHANGE_TOKEN);
 		/* totsize = type + len + string + \0 */
 		totsize = strlen(newvalue) + 3;
 		tds_put_smallint(tds, totsize);
@@ -59,9 +71,8 @@ tds_env_change(TDSSOCKET * tds, int type, const char *oldvalue, const char *newv
 		tds_put_n(tds, newvalue, strlen(newvalue));
 		tds_put_byte(tds, '\0');
 		break;
-		/* code page would be 3 */
-		/* packet size */
-	case 4:
+	case TDS_ENV_PACKSIZE:
+		tds_put_byte(tds, TDS_ENVCHANGE_TOKEN);
 		/* totsize = type + len + string + \0 */
 		totsize = strlen(newvalue) + 3;
 		tds_put_smallint(tds, totsize);
@@ -70,6 +81,23 @@ tds_env_change(TDSSOCKET * tds, int type, const char *oldvalue, const char *newv
 		tds_put_n(tds, newvalue, strlen(newvalue));
 		tds_put_byte(tds, '\0');
 		break;
+	case TDS_ENV_LCID:
+	case TDS_ENV_SQLCOLLATION:
+	case TDS_ENV_CHARSET:
+#if 1
+		tds_put_byte(tds, TDS_ENVCHANGE_TOKEN);
+		/* totsize = type + len + oldvalue + len + newvalue */
+		totsize = 3 + strlen(newvalue) + strlen(oldvalue);
+		tds_put_smallint(tds, totsize);
+		tds_put_byte(tds, type);
+		tds_put_byte(tds, strlen(newvalue));
+		tds_put_n(tds, newvalue, strlen(newvalue));
+		tds_put_byte(tds, strlen(oldvalue));
+		tds_put_n(tds, oldvalue, strlen(oldvalue));
+		break;
+#endif
+	default:
+		tdsdump_log(TDS_DBG_WARN, "tds_env_change() ignoring unsupported environment code #%d", type);
 	}
 }
 
@@ -185,21 +213,43 @@ tds_send_capabilities_token(TDSSOCKET * tds)
 	tds_put_byte(tds, 0);
 }
 
+/**
+ * Send a "done" token, marking the end of a table, stored procedure, or query.
+ * \param tds		Where the token will be written to.
+ * \param token		The appropriate type of "done" token for this context:
+ *			TDS_DONE_TOKEN outside a stored procedure,
+ *			TDS_DONEINPROC_TOKEN inside a stored procedure, or
+ *			TDS_DONEPROC_TOKEN at the end of a stored procedure.
+ * \param flags		Bitwise-OR of flags in the "enum tds_end" data type.
+ *			TDS_DONE_FINAL for the last statement in a query,
+ *			TDS_DONE_MORE_RESULTS if not the last statement,
+ *			TDS_DONE_ERROR if the statement had an error,
+ *			TDS_DONE_INXACT if a transaction is  pending,
+ *			TDS_DONE_PROC inside a stored procedure,
+ *			TDS_DONE_COUNT if a table was sent (and rows counted)
+ *			TDS_DONE_CANCELLED if the query was canceled, and
+ *			TDS_DONE_EVENT if the token marks an event.
+ * \param numrows	Number of rows, if flags has TDS_DONE_COUNT.
+ */
 void
-tds_send_253_token(TDSSOCKET * tds, TDS_TINYINT flags, TDS_INT numrows)
+tds_send_done(TDSSOCKET * tds, int token, TDS_SMALLINT flags, TDS_INT numrows)
 {
-	tds_put_byte(tds, 253);
-	tds_put_byte(tds, flags);
-	tds_put_byte(tds, 0);
-	tds_put_byte(tds, 2);
-	tds_put_byte(tds, 0);
+	tds_put_byte(tds, token);
+	tds_put_smallint(tds, flags);
+	tds_put_smallint(tds, 2); /* are these two bytes the transaction status? */
 	tds_put_int(tds, numrows);
 }
 
+void
+tds_send_253_token(TDSSOCKET * tds, TDS_SMALLINT flags, TDS_INT numrows)
+{
+	tds_send_done(tds, 253, flags, numrows);
+}
+
 void
 tds_send_174_token(TDSSOCKET * tds, TDS_SMALLINT numcols)
 {
-int i;
+	int i;
 
 	tds_put_byte(tds, 174);
 	tds_put_smallint(tds, numcols);
@@ -210,28 +260,29 @@ int i;
 void
 tds_send_col_name(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 {
-int col, hdrsize = 0;
-TDSCOLINFO *curcol;
+	int col, hdrsize = 0;
+	TDSCOLUMN *curcol;
 
 	tds_put_byte(tds, TDS_COLNAME_TOKEN);
 	for (col = 0; col < resinfo->num_cols; col++) {
 		curcol = resinfo->columns[col];
-		hdrsize += strlen(curcol->column_name) + 2;
+		assert(strlen(curcol->column_name) == curcol->column_namelen);
+		hdrsize += curcol->column_namelen + 1;
 	}
 
 	tds_put_smallint(tds, hdrsize);
 	for (col = 0; col < resinfo->num_cols; col++) {
 		curcol = resinfo->columns[col];
-		tds_put_byte(tds, strlen(curcol->column_name));
-		/* include the null */
-		tds_put_n(tds, curcol->column_name, strlen(curcol->column_name) + 1);
+		tds_put_byte(tds, curcol->column_namelen);
+		/* exclude the null */
+		tds_put_n(tds, curcol->column_name, curcol->column_namelen);
 	}
 }
 void
 tds_send_col_info(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 {
-int col, hdrsize = 0;
-TDSCOLINFO *curcol;
+	int col, hdrsize = 0;
+	TDSCOLUMN *curcol;
 
 	tds_put_byte(tds, TDS_COLFMT_TOKEN);
 
@@ -257,15 +308,16 @@ TDSCOLINFO *curcol;
 void
 tds_send_result(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 {
-TDSCOLINFO *curcol;
-int i, totlen;
+	TDSCOLUMN *curcol;
+	int i, totlen;
 
 	tds_put_byte(tds, TDS_RESULT_TOKEN);
 	totlen = 2;
 	for (i = 0; i < resinfo->num_cols; i++) {
 		curcol = resinfo->columns[i];
 		totlen += 8;
-		totlen += strlen(curcol->column_name);
+		assert(strlen(curcol->column_name) == curcol->column_namelen);
+		totlen += curcol->column_namelen;
 		curcol = resinfo->columns[i];
 		if (!is_fixed_type(curcol->column_type)) {
 			totlen++;
@@ -275,8 +327,8 @@ int i, totlen;
 	tds_put_smallint(tds, resinfo->num_cols);
 	for (i = 0; i < resinfo->num_cols; i++) {
 		curcol = resinfo->columns[i];
-		tds_put_byte(tds, strlen(curcol->column_name));
-		tds_put_n(tds, curcol->column_name, strlen(curcol->column_name));
+		tds_put_byte(tds, curcol->column_namelen);
+		tds_put_n(tds, curcol->column_name, curcol->column_namelen);
 		tds_put_byte(tds, '0');
 		tds_put_int(tds, curcol->column_usertype);
 		tds_put_byte(tds, curcol->column_type);
@@ -287,11 +339,102 @@ int i, totlen;
 	}
 }
 
+void
+tds7_send_result(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
+{
+	int i, j;
+	TDSCOLUMN *curcol;
+
+	/* TDS7+ uses TDS7_RESULT_TOKEN to send column names and info */
+	tds_put_byte(tds, TDS7_RESULT_TOKEN);
+
+	/* send the number of columns */
+	tds_put_smallint(tds, resinfo->num_cols);
+
+	/* send info about each column */
+	for (i = 0; i < resinfo->num_cols; i++) {
+
+		/* usertype, flags, and type */
+		curcol = resinfo->columns[i];
+		tds_put_smallint(tds, curcol->column_usertype);
+		tds_put_smallint(tds, curcol->column_flags);
+		tds_put_byte(tds, curcol->column_type); /* smallint? */
+
+		/* bytes in "size" field varies */
+		if (is_blob_type(curcol->column_type)) {
+			tds_put_int(tds, curcol->column_size);
+		} else if (curcol->column_type>=128) { /*is_large_type*/
+			tds_put_smallint(tds, curcol->column_size);
+		} else {
+			tds_put_tinyint(tds, curcol->column_size);
+		}
+
+		/* some types have extra info */
+		if (is_numeric_type(curcol->column_type)) {
+			tds_put_tinyint(tds, curcol->column_prec);
+			tds_put_tinyint(tds, curcol->column_scale);
+		} else if (is_blob_type(curcol->column_type)) {
+			tds_put_smallint(tds, 2 * strlen(curcol->table_name));
+			for (j = 0; curcol->table_name[j] != '\0'; j++){
+				tds_put_byte(tds, curcol->table_name[j]);
+				tds_put_byte(tds, 0);
+			}
+		}
+
+		/* finally the name, in UCS16 format */
+		assert(strlen(curcol->column_name) == curcol->column_namelen);
+		tds_put_byte(tds, curcol->column_namelen);
+		for (j = 0; j < curcol->column_namelen; j++) {
+			tds_put_byte(tds, curcol->column_name[j]);
+			tds_put_byte(tds, 0);
+		}
+	}
+}
+
+/**
+ * Send any tokens that mark the start of a table.  This automatically chooses
+ * the right tokens for this client's version of the TDS protocol.  In other
+ * words, it is a wrapper around tds_send_col_name(), tds_send_col_info(),
+ * tds_send_result(), and tds7_send_result().
+ * \param tds		The socket to which the tokens will be written.  Also,
+ *			it contains the TDS protocol version number.
+ * \param resinfo	Describes the table to be send, especially the number
+ *			of columns and the names & data types of each column.
+ */
+void tds_send_table_header(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
+{
+	switch (tds->major_version) {
+	case 4:
+		/*
+		 * TDS4 uses TDS_COLNAME_TOKEN to send column names, and
+		 * TDS_COLFMT_TOKEN to send column info.  The number of columns
+		 * is implied by the number of column names.
+		 */
+		tds_send_col_name(tds, resinfo);
+		tds_send_col_info(tds, resinfo);
+		break;
+
+	case 5:
+		/* TDS5 uses a TDS_RESULT_TOKEN to send all column information */
+		tds_send_result(tds, resinfo);
+		break;
+
+	case 7:
+	case 8:
+		/*
+		 * TDS7+ uses a TDS7_RESULT_TOKEN to send all column
+		 * information.
+		 */
+		tds7_send_result(tds, resinfo);
+		break;
+	}
+}
+
 void
 tds_send_row(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 {
-TDSCOLINFO *curcol;
-int colsize, i;
+	TDSCOLUMN *curcol;
+	int colsize, i;
 
 	tds_put_byte(tds, TDS_ROW_TOKEN);
 	for (i = 0; i < resinfo->num_cols; i++) {
