@@ -33,9 +33,46 @@
 
 #include "tds.h"
 
+#ifdef __linux__
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <linux/if.h>
+#endif
+
 /* TODO get real MAC */
 void
 tds_getmac(int s, unsigned char mac[6])
 {
+/* implementation for Linux */
+#ifdef __linux__
+	struct ifreq ifr;
+	struct ifreq *IFR;
+	struct ifconf ifc;
+	char buf[1024];
+	int i;
+
 	memset(mac, 0, 6);
+
+	ifc.ifc_len = sizeof(buf);
+	ifc.ifc_buf = buf;
+	ioctl(s, SIOCGIFCONF, &ifc);
+
+	IFR = ifc.ifc_req;
+	for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ++IFR) {
+
+		strcpy(ifr.ifr_name, IFR->ifr_name);
+		if (ioctl(s, SIOCGIFFLAGS, &ifr) == 0) {
+			if (!(ifr.ifr_flags & IFF_LOOPBACK)) {
+				if (ioctl(s, SIOCGIFHWADDR, &ifr) == 0) {
+					memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
+					break;
+				}
+			}
+		}
+	}
+#else /* !defined(__linux__) */
+	memset(mac, 0, 6);
+#endif
 }
