@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -43,6 +43,7 @@
 #endif /* HAVE_STDLIB_H */
 
 #include <stdio.h>
+#include <assert.h>
 
 #if HAVE_STRING_H
 #include <string.h>
@@ -60,6 +61,10 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
+#if HAVE_NETINET_TCP_H
+#include <netinet/tcp.h>
+#endif /* HAVE_NETINET_TCP_H */
+
 #if HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif /* HAVE_ARPA_INET_H */
@@ -72,18 +77,23 @@
 #include <sys/ioctl.h>
 #endif /* HAVE_SYS_IOCTL_H */
 
+#if HAVE_SELECT_H
+#include <sys/select.h>
+#endif /* HAVE_SELECT_H */
+
 #include "tds.h"
+#include "tdsiconv.h"
 #include "tdsstring.h"
 #include "replacements.h"
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: login.c,v 1.82 2003/01/10 20:04:21 freddy77 Exp $";
+static char software_version[] = "$Id: login.c,v 1.126 2004/07/29 10:22:41 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
-static int tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info);
-static int tds7_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info);
+static int tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
+static int tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
 
 void
 tds_set_version(TDSLOGIN * tds_login, short major_ver, short minor_ver)
@@ -155,9 +165,9 @@ tds_set_library(TDSLOGIN * tds_login, const char *library)
 }
 
 void
-tds_set_charset(TDSLOGIN * tds_login, const char *charset)
+tds_set_client_charset(TDSLOGIN * tds_login, const char *charset)
 {
-	tds_dstr_copy(&tds_login->char_set, charset);
+	tds_dstr_copy(&tds_login->client_charset, charset);
 }
 
 void
@@ -166,17 +176,8 @@ tds_set_language(TDSLOGIN * tds_login, const char *language)
 	tds_dstr_copy(&tds_login->language, language);
 }
 
-/* Jeffs' hack to support timeouts */
 void
-tds_set_timeouts(TDSLOGIN * tds_login, int connect_timeout, int query_timeout, int longquery_timeout)
-{
-	tds_login->connect_timeout = connect_timeout;
-	tds_login->query_timeout = query_timeout;
-	tds_login->longquery_timeout = longquery_timeout;
-}
-
-void
-tds_set_longquery_handler(TDSLOGIN * tds_login, void (*longquery_func) (long), long longquery_param)
+tds_set_longquery_handler(TDSLOGIN * tds_login, void (*longquery_func) (void *), void *longquery_param)
 {				/* Jeff's hack */
 	tds_login->longquery_func = longquery_func;
 	tds_login->longquery_param = longquery_param;
@@ -192,11 +193,11 @@ tds_set_capabilities(TDSLOGIN * tds_login, unsigned char *capabilities, int size
  * Do a connection to socket
  * @param tds connection structure. This should be a non-connected connection.
  *        If the function fail connection is deleted
- * @param connect_info info for connection
+ * @param connection info for connection
  * @return TDS_FAIL or TDS_SUCCEED
  */
 int
-tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
+tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 {
 	struct sockaddr_in sin;
 
@@ -211,22 +212,21 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 	char version[256];
 	char *str;
 	int len;
-	TDS_INT restype;
 
 	FD_ZERO(&fds);
 
 	/*
 	 * If a dump file has been specified, start logging
 	 */
-	if (!tds_dstr_isempty(&connect_info->dump_file)) {
-		tdsdump_open(connect_info->dump_file);
+	if (!tds_dstr_isempty(&connection->dump_file)) {
+		tdsdump_open(tds_dstr_cstr(&connection->dump_file));
 	}
 
-	tds->connect_info = connect_info;
+	tds->connection = connection;
 
-	tds->major_version = connect_info->major_version;
-	tds->minor_version = connect_info->minor_version;
-	tds->emul_little_endian = connect_info->emul_little_endian;
+	tds->major_version = connection->major_version;
+	tds->minor_version = connection->minor_version;
+	tds->emul_little_endian = connection->emul_little_endian;
 #ifdef WORDS_BIGENDIAN
 	if (IS_TDS7_PLUS(tds)) {
 		/* TDS 7/8 only supports little endian */
@@ -235,61 +235,74 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 #endif
 
 	/* set up iconv */
-	if (connect_info->client_charset) {
-		tds_iconv_open(tds, connect_info->client_charset);
+	if (connection->client_charset) {
+		tds_iconv_open(tds, tds_dstr_cstr(&connection->client_charset));
 	}
 
 	/* specified a date format? */
 	/*
-	 * if (connect_info->date_fmt) {
-	 * tds->date_fmt=strdup(connect_info->date_fmt);
+	 * if (connection->date_fmt) {
+	 * tds->date_fmt=strdup(connection->date_fmt);
 	 * }
 	 */
-	connect_timeout = connect_info->connect_timeout;
+	connect_timeout = connection->connect_timeout;
 
 	/* Jeff's hack - begin */
-	tds->timeout = (connect_timeout) ? connect_info->query_timeout : 0;
-	tds->longquery_timeout = (connect_timeout) ? connect_info->longquery_timeout : 0;
-	tds->longquery_func = connect_info->longquery_func;
-	tds->longquery_param = connect_info->longquery_param;
+	tds->timeout = (connect_timeout) ? connection->query_timeout : 0;
+	tds->longquery_timeout = (connect_timeout) ? connection->longquery_timeout : 0;
+	tds->longquery_func = connection->longquery_func;
+	tds->longquery_param = connection->longquery_param;
 	/* end */
 
 	/* verify that ip_addr is not NULL */
-	if (tds_dstr_isempty(&connect_info->ip_addr)) {
-		tdsdump_log(TDS_DBG_ERROR, "%L IP address pointer is NULL\n");
-		if (connect_info->server_name) {
-			tdsdump_log(TDS_DBG_ERROR, "%L Server %s not found!\n", connect_info->server_name);
+	if (tds_dstr_isempty(&connection->ip_addr)) {
+		tdsdump_log(TDS_DBG_ERROR, "IP address pointer is NULL\n");
+		if (connection->server_name) {
+			tdsdump_log(TDS_DBG_ERROR, "Server %s not found!\n", tds_dstr_cstr(&connection->server_name));
 		} else {
-			tdsdump_log(TDS_DBG_ERROR, "%L No server specified!\n");
+			tdsdump_log(TDS_DBG_ERROR, "No server specified!\n");
 		}
 		tds_free_socket(tds);
 		return TDS_FAIL;
 	}
-	sin.sin_addr.s_addr = inet_addr(connect_info->ip_addr);
-	if (sin.sin_addr.s_addr == -1) {
-		tdsdump_log(TDS_DBG_ERROR, "%L inet_addr() failed, IP = %s\n", connect_info->ip_addr);
+	sin.sin_addr.s_addr = inet_addr(tds_dstr_cstr(&connection->ip_addr));
+	if (sin.sin_addr.s_addr == INADDR_NONE) {
+		tdsdump_log(TDS_DBG_ERROR, "inet_addr() failed, IP = %s\n", tds_dstr_cstr(&connection->ip_addr));
 		tds_free_socket(tds);
 		return TDS_FAIL;
 	}
 
 	sin.sin_family = AF_INET;
-	sin.sin_port = htons(connect_info->port);
+	sin.sin_port = htons(connection->port);
 
-	memcpy(tds->capabilities, connect_info->capabilities, TDS_MAX_CAPABILITY);
+	memcpy(tds->capabilities, connection->capabilities, TDS_MAX_CAPABILITY);
 
 
 	retval = tds_version(tds, version);
 	if (!retval)
 		version[0] = '\0';
 
-	tdsdump_log(TDS_DBG_INFO1, "%L Connecting addr %s port %d with TDS version %s\n", inet_ntoa(sin.sin_addr),
-		    ntohs(sin.sin_port), version);
-	if ((tds->s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	tdsdump_log(TDS_DBG_INFO1, "Connecting to %s port %d, TDS %s.\n", inet_ntoa(sin.sin_addr), ntohs(sin.sin_port), version);
+	if (TDS_IS_SOCKET_INVALID(tds->s = socket(AF_INET, SOCK_STREAM, 0))) {
 		perror("socket");
 		tds_free_socket(tds);
 		return TDS_FAIL;
 	}
 
+#ifdef SO_KEEPALIVE
+	len = 1;
+	setsockopt(tds->s, SOL_SOCKET, SO_KEEPALIVE, (const void *) &len, sizeof(len));
+#endif
+
+#if defined(TCP_NODELAY) && (defined(IPPROTO_TCP) || defined(SOL_TCP))
+	len = 1;
+#ifdef SOL_TCP
+	setsockopt(tds->s, SOL_TCP, TCP_NODELAY, (const void *) &len, sizeof(len));
+#else
+	setsockopt(tds->s, IPPROTO_TCP, TCP_NODELAY, (const void *) &len, sizeof(len));
+#endif
+#endif
+	
 	/* Jeff's hack *** START OF NEW CODE *** */
 	if (connect_timeout) {
 		start = time(NULL);
@@ -299,7 +312,7 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 			return TDS_FAIL;
 		}
 		retval = connect(tds->s, (struct sockaddr *) &sin, sizeof(sin));
-		if (retval < 0 && errno == EINPROGRESS)
+		if (retval < 0 && sock_errno == TDSSOCK_EINPROGRESS)
 			retval = 0;
 		if (retval < 0) {
 			perror("src/tds/login.c: tds_connect (timed)");
@@ -313,7 +326,7 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 			selecttimeout.tv_sec = connect_timeout - (now - start);
 			selecttimeout.tv_usec = 0;
 			retval = select(tds->s + 1, NULL, &fds, NULL, &selecttimeout);
-			if (retval < 0 && errno == EINTR)
+			if (retval < 0 && sock_errno == TDSSOCK_EINTR)
 				retval = 0;
 			now = time(NULL);
 		}
@@ -338,11 +351,11 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 
 	if (IS_TDS7_PLUS(tds)) {
 		tds->out_flag = 0x10;
-		tds7_send_login(tds, connect_info);
+		tds7_send_login(tds, connection);
 		db_selected = 1;
 	} else {
 		tds->out_flag = 0x02;
-		tds_send_login(tds, connect_info);
+		tds_send_login(tds, connection);
 	}
 	if (!tds_process_login_tokens(tds)) {
 		tds_close_socket(tds);
@@ -351,16 +364,19 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 		return TDS_FAIL;
 	}
 
-	if (connect_info->text_size || (!db_selected && !tds_dstr_isempty(&connect_info->database))) {
-		len = 64 + tds_quote_id(tds, NULL, connect_info->database);
-		str = (char*) malloc(len);
-		str[0] = 0;
-		if (connect_info->text_size) {
-			sprintf(str, "set textsize %d ", connect_info->text_size);
+	if (connection->text_size || (!db_selected && !tds_dstr_isempty(&connection->database))) {
+		len = 64 + tds_quote_id(tds, NULL, tds_dstr_cstr(&connection->database),-1);
+		if ((str = (char *) malloc(len)) == NULL) {
+			tds_free_socket(tds);
+			return TDS_FAIL;
 		}
-		if (!db_selected && !tds_dstr_isempty(&connect_info->database)) {
+		str[0] = 0;
+		if (connection->text_size) {
+			sprintf(str, "set textsize %d ", connection->text_size);
+		}
+		if (!db_selected && !tds_dstr_isempty(&connection->database)) {
 			strcat(str, "use ");
-			tds_quote_id(tds, strchr(str, 0), connect_info->database);
+			tds_quote_id(tds, strchr(str, 0), tds_dstr_cstr(&connection->database), -1);
 		}
 		retval = tds_submit_query(tds, str);
 		free(str);
@@ -369,13 +385,13 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 			return TDS_FAIL;
 		}
 
-		if (tds_process_simple_query(tds, &restype) == TDS_FAIL || restype == TDS_CMD_FAIL) {
+		if (tds_process_simple_query(tds) != TDS_SUCCEED) {
 			tds_free_socket(tds);
 			return TDS_FAIL;
 		}
 	}
 
-	tds->connect_info = NULL;
+	tds->connection = NULL;
 	return TDS_SUCCEED;
 }
 
@@ -388,7 +404,7 @@ tds_put_login_string(TDSSOCKET * tds, const char *buf, int n)
 }
 
 static int
-tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
+tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 {
 #ifdef WORDS_BIGENDIAN
 	static const unsigned char be1[] = { 0x02, 0x00, 0x06, 0x04, 0x08, 0x01 };
@@ -413,7 +429,6 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 	 */
 	static const unsigned char magic5[] = { 0x00, 0x00 };
 	static const unsigned char magic6[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
-	static const unsigned char magic7 = 0x01;
 
 	static const unsigned char magic42[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 	static const unsigned char magic50[] = { 0x00, 0x00, 0x00, 0x00 };
@@ -435,6 +450,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 
 	unsigned char protocol_version[4];
 	unsigned char program_version[4];
+	const char *server_charset;
 
 	int len;
 	char blockstr[16];
@@ -458,9 +474,9 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 	 * do this, (well...mine was a kludge actually) so here's mostly his
 	 */
 
-	tds_put_login_string(tds, connect_info->host_name, TDS_MAX_LOGIN_STR_SZ);	/* client host name */
-	tds_put_login_string(tds, connect_info->user_name, TDS_MAX_LOGIN_STR_SZ);	/* account name */
-	tds_put_login_string(tds, connect_info->password, TDS_MAX_LOGIN_STR_SZ);	/* account password */
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->host_name), TDS_MAX_LOGIN_STR_SZ);	/* client host name */
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->user_name), TDS_MAX_LOGIN_STR_SZ);	/* account name */
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->password), TDS_MAX_LOGIN_STR_SZ);	/* account password */
 	tds_put_login_string(tds, "37876", TDS_MAX_LOGIN_STR_SZ);	/* host process */
 #ifdef WORDS_BIGENDIAN
 	if (tds->emul_little_endian) {
@@ -471,7 +487,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 #else
 	tds_put_n(tds, le1, 6);
 #endif
-	tds_put_byte(tds, connect_info->bulk_copy);
+	tds_put_byte(tds, connection->bulk_copy);
 	tds_put_n(tds, magic2, 2);
 	if (IS_TDS42(tds)) {
 		tds_put_int(tds, 512);
@@ -479,23 +495,23 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 		tds_put_int(tds, 0);
 	}
 	tds_put_n(tds, magic3, 3);
-	tds_put_login_string(tds, connect_info->app_name, TDS_MAX_LOGIN_STR_SZ);
-	tds_put_login_string(tds, connect_info->server_name, TDS_MAX_LOGIN_STR_SZ);
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->app_name), TDS_MAX_LOGIN_STR_SZ);
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->server_name), TDS_MAX_LOGIN_STR_SZ);
 	if (IS_TDS42(tds)) {
-		tds_put_login_string(tds, connect_info->password, 255);
+		tds_put_login_string(tds, tds_dstr_cstr(&connection->password), 255);
 	} else {
-		len = strlen(connect_info->password);
+		len = tds_dstr_len(&connection->password);
 		if (len > 253)
 			len = 0;
 		tds_put_byte(tds, 0);
 		tds_put_byte(tds, len);
-		tds_put_n(tds, connect_info->password, len);
+		tds_put_n(tds, connection->password, len);
 		tds_put_n(tds, NULL, 253 - len);
 		tds_put_byte(tds, len + 2);
 	}
 
 	tds_put_n(tds, protocol_version, 4);	/* TDS version; { 0x04,0x02,0x00,0x00 } */
-	tds_put_login_string(tds, connect_info->library, 10);	/* client program name */
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->library), 10);	/* client program name */
 	if (IS_TDS42(tds)) {
 		tds_put_int(tds, 0);
 	} else {
@@ -510,17 +526,28 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 #else
 	tds_put_n(tds, le2, 3);
 #endif
-	tds_put_login_string(tds, connect_info->language, TDS_MAX_LOGIN_STR_SZ);	/* language */
-	tds_put_byte(tds, connect_info->suppress_language);
+	tds_put_login_string(tds, tds_dstr_cstr(&connection->language), TDS_MAX_LOGIN_STR_SZ);	/* language */
+	tds_put_byte(tds, connection->suppress_language);
 	tds_put_n(tds, magic5, 2);
-	tds_put_byte(tds, connect_info->encrypted);
+	tds_put_byte(tds, connection->encrypted);
 	tds_put_n(tds, magic6, 10);
-	tds_put_login_string(tds, connect_info->char_set, TDS_MAX_LOGIN_STR_SZ);	/* charset */
-	tds_put_byte(tds, magic7);
+
+	/* use charset nearest to client or nothing */
+	server_charset = NULL;
+	if (!tds_dstr_isempty(&connection->server_charset))
+		server_charset = tds_dstr_cstr(&connection->server_charset);
+	else
+		server_charset = tds_sybase_charset_name(tds_dstr_cstr(&connection->client_charset));
+	if (!server_charset)
+		server_charset = "";
+	tds_put_login_string(tds, server_charset, TDS_MAX_LOGIN_STR_SZ);	/* charset */
+	/* this is a flag, mean that server should use character set provided by client */
+	/* TODO notify charset change ?? what's correct meaning ?? -- freddy77 */
+	tds_put_byte(tds, 1);
 
 	/* network packet size */
-	if (connect_info->block_size < 1000000)
-		sprintf(blockstr, "%d", connect_info->block_size);
+	if (connection->block_size < 1000000 && connection->block_size)
+		sprintf(blockstr, "%d", connection->block_size);
 	else
 		strcpy(blockstr, "512");
 	tds_put_login_string(tds, blockstr, 6);
@@ -555,19 +582,19 @@ tds7_send_auth(TDSSOCKET * tds, const unsigned char *challenge)
 	int password_len;
 	int domain_len;
 
-	TDSCONNECTINFO *connect_info = tds->connect_info;
+	TDSCONNECTION *connection = tds->connection;
 
-	/* check connect_info */
-	if (!connect_info)
+	/* check connection */
+	if (!connection)
 		return TDS_FAIL;
 
 	/* parse a bit of config */
-	domain = connect_info->default_domain;
-	user_name = connect_info->user_name;
+	domain = tds_dstr_cstr(&connection->default_domain);
+	user_name = tds_dstr_cstr(&connection->user_name);
 	user_name_len = user_name ? strlen(user_name) : 0;
-	host_name_len = connect_info->host_name ? strlen(connect_info->host_name) : 0;
-	password_len = connect_info->password ? strlen(connect_info->password) : 0;
-	domain_len = domain ? strlen(domain) : 0;
+	host_name_len = tds_dstr_len(&connection->host_name);
+	password_len = tds_dstr_len(&connection->password);
+	domain_len = strlen(domain);
 
 	/* check override of domain */
 	if (user_name && (p = strchr(user_name, '\\')) != NULL) {
@@ -623,9 +650,9 @@ tds7_send_auth(TDSSOCKET * tds, const unsigned char *challenge)
 
 	tds_put_string(tds, domain, domain_len);
 	tds_put_string(tds, user_name, user_name_len);
-	tds_put_string(tds, connect_info->host_name, host_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->host_name), host_name_len);
 
-	tds_answer_challenge(connect_info->password, challenge, &answer);
+	tds_answer_challenge(tds_dstr_cstr(&connection->password), challenge, &answer);
 	tds_put_n(tds, answer.lm_resp, 24);
 	tds_put_n(tds, answer.nt_resp, 24);
 
@@ -638,55 +665,51 @@ tds7_send_auth(TDSSOCKET * tds, const unsigned char *challenge)
 /**
  * tds7_send_login() -- Send a TDS 7.0 login packet
  * TDS 7.0 login packet is vastly different and so gets its own function
+ * \returns the return value is ignored by the caller. :-/
  */
 static int
-tds7_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
+tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 {
 	int rc;
-	static const unsigned char magic1_domain[] = { 6, 0x7d, 0x0f, 0xfd,
-		0xff, 0x0, 0x0, 0x0,	/* Client PID */
-		/* the 0x80 in the third byte controls whether this is a domain login 
-		 * or not  0x80 = yes, 0x00 = no */
-		0x0, 0xe0, 0x83, 0x0,	/* Connection ID of the Primary Server (?) */
-		0x0,			/* Option Flags 1 */
-		0x68,			/* Option Flags 2 */
-		0x01,
-		0x00,
-		0x00, 0x09, 0x04, 0x00,
-		0x00
-	};
-	static const unsigned char magic1_server[] = { 6, 0x83, 0xf2, 0xf8,	/* Client Program version */
-		0xff, 0x0, 0x0, 0x0,	/* Client PID */
-		0x0, 0xe0, 0x03, 0x0,	/* Connection ID of the Primary Server (?) */
-		0x0,			/* Option Flags 1 */
-		0x88,			/* Option Flags 2 */
-		0xff,			/* Type Flags     */
-		0xff,			/* reserved Flags */
-		0xff, 0x36, 0x04, 0x00,
-		0x00
-	};
-	unsigned const char *magic1 = magic1_server;
+
+	static const unsigned char client_progver[] = { 6, 0x83, 0xf2, 0xf8 };
+
+	static const unsigned char tds7Version[] = { 0x00, 0x00, 0x00, 0x70 };
+	static const unsigned char tds8Version[] = { 0x01, 0x00, 0x00, 0x71 };
+
+	static const unsigned char connection_id[] = { 0x00, 0x00, 0x00, 0x00 };
+	unsigned char option_flag1 = 0x00;
+	unsigned char option_flag2 = tds->option_flag2;
+	static const unsigned char sql_type_flag = 0x00;
+	static const unsigned char reserved_flag = 0x00;
+
+	static const unsigned char time_zone[] = { 0x88, 0xff, 0xff, 0xff };
+	static const unsigned char collation[] = { 0x36, 0x04, 0x00, 0x00 };
+
 	unsigned char hwaddr[6];
 
 	/* 0xb4,0x00,0x30,0x00,0xe4,0x00,0x00,0x00; */
 	char unicode_string[256];
+	char *punicode;
+	size_t unicode_left;
 	int packet_size;
+	int block_size;
 	int current_pos;
 	static const unsigned char ntlm_id[] = "NTLMSSP";
-	int domain_login = connect_info->try_domain_login ? 1 : 0;
+	int domain_login = connection->try_domain_login ? 1 : 0;
 
-	const char *domain = connect_info->default_domain;
-	const char *user_name = connect_info->user_name;
+	const char *domain = tds_dstr_cstr(&connection->default_domain);
+	const char *user_name = tds_dstr_cstr(&connection->user_name);
 	const char *p;
-	int user_name_len = user_name ? strlen(user_name) : 0;
-	int host_name_len = connect_info->host_name ? strlen(connect_info->host_name) : 0;
-	int app_name_len = connect_info->app_name ? strlen(connect_info->app_name) : 0;
-	int password_len = connect_info->password ? strlen(connect_info->password) : 0;
-	int server_name_len = connect_info->server_name ? strlen(connect_info->server_name) : 0;
-	int library_len = connect_info->library ? strlen(connect_info->library) : 0;
-	int language_len = connect_info->language ? strlen(connect_info->language) : 0;
-	int database_len = connect_info->database ? strlen(connect_info->database) : 0;
-	int domain_len = domain ? strlen(domain) : 0;
+	int user_name_len = strlen(user_name);
+	int host_name_len = tds_dstr_len(&connection->host_name);
+	int app_name_len = tds_dstr_len(&connection->app_name);
+	size_t password_len = tds_dstr_len(&connection->password);
+	int server_name_len = tds_dstr_len(&connection->server_name);
+	int library_len = tds_dstr_len(&connection->library);
+	int language_len = tds_dstr_len(&connection->language);
+	int database_len = tds_dstr_len(&connection->database);
+	int domain_len = strlen(domain);
 	int auth_len = 0;
 
 	/* avoid overflow limiting password */
@@ -706,22 +729,46 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 
 	packet_size = 86 + (host_name_len + app_name_len + server_name_len + library_len + language_len + database_len) * 2;
 	if (domain_login) {
-		magic1 = magic1_domain;
 		auth_len = 32 + host_name_len + domain_len;
 		packet_size += auth_len;
 	} else
 		packet_size += (user_name_len + password_len) * 2;
 
-	tds_put_smallint(tds, packet_size);
-	tds_put_n(tds, NULL, 5);
+	tds_put_int(tds, packet_size);
 	if (IS_TDS80(tds)) {
-		tds_put_byte(tds, 0x80);
+		tds_put_n(tds, tds8Version, 4);
 	} else {
-		tds_put_byte(tds, 0x70);
+		tds_put_n(tds, tds7Version, 4);
 	}
-	tds_put_n(tds, NULL, 3);	/* rest of TDSVersion which is a 4 byte field    */
-	tds_put_n(tds, NULL, 4);	/* desired packet size being requested by client */
-	tds_put_n(tds, magic1, 21);
+
+	if (connection->block_size < 1000000)
+		block_size = connection->block_size;
+	else
+		block_size = 4096;	/* SQL server default */
+	tds_put_int(tds, block_size);	/* desired packet size being requested by client */
+
+	tds_put_n(tds, client_progver, 4);	/* client program version ? */
+
+	tds_put_int(tds, getpid());	/* process id of this process */
+
+	tds_put_n(tds, connection_id, 4);
+
+	option_flag1 |= 0x80;	/* enable warning messages if SET LANGUAGE issued   */
+	option_flag1 |= 0x40;	/* change to initial database must succeed          */
+	option_flag1 |= 0x20;	/* enable warning messages if USE <database> issued */
+
+	tds_put_byte(tds, option_flag1);
+
+	if (domain_login)
+		option_flag2 |= 0x80;	/* enable domain login security                     */
+
+	tds_put_byte(tds, option_flag2);
+
+	tds_put_byte(tds, sql_type_flag);
+	tds_put_byte(tds, reserved_flag);
+
+	tds_put_n(tds, time_zone, 4);
+	tds_put_n(tds, collation, 4);
 
 	current_pos = 86;	/* ? */
 	/* host name */
@@ -783,18 +830,29 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 	tds_put_smallint(tds, current_pos);
 	tds_put_smallint(tds, 0);
 
-	tds_put_string(tds, connect_info->host_name, host_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->host_name), host_name_len);
 	if (!domain_login) {
-		tds_put_string(tds, connect_info->user_name, user_name_len);
-		tds7_ascii2unicode(tds, connect_info->password, unicode_string, 256);
-		tds7_crypt_pass((unsigned char *) unicode_string, password_len * 2, (unsigned char *) unicode_string);
-		tds_put_n(tds, unicode_string, password_len * 2);
+		TDSICONV *char_conv = tds->char_convs[client2ucs2];
+		tds_put_string(tds, tds_dstr_cstr(&connection->user_name), user_name_len);
+		p = tds_dstr_cstr(&connection->password);
+		punicode = unicode_string;
+		unicode_left = sizeof(unicode_string);
+
+		memset(&char_conv->suppress, 0, sizeof(char_conv->suppress));
+		if (tds_iconv(tds, tds->char_convs[client2ucs2], to_server, &p, &password_len, &punicode, &unicode_left) ==
+		    (size_t) - 1) {
+			tdsdump_log(TDS_DBG_INFO1, "password \"%s\" could not be converted to UCS-2\n", p);
+			assert(0);
+		}
+		password_len = punicode - unicode_string;
+		tds7_crypt_pass((unsigned char *) unicode_string, password_len, (unsigned char *) unicode_string);
+		tds_put_n(tds, unicode_string, password_len);
 	}
-	tds_put_string(tds, connect_info->app_name, app_name_len);
-	tds_put_string(tds, connect_info->server_name, server_name_len);
-	tds_put_string(tds, connect_info->library, library_len);
-	tds_put_string(tds, connect_info->language, language_len);
-	tds_put_string(tds, connect_info->database, database_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->app_name), app_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->server_name), server_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->library), library_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->language), language_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->database), database_len);
 
 	if (domain_login) {
 		/* from here to the end of the packet is the NTLMSSP authentication */
@@ -815,7 +873,7 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTINFO * connect_info)
 		tds_put_int(tds, 32);
 
 		/* hostname and domain */
-		tds_put_n(tds, connect_info->host_name, host_name_len);
+		tds_put_n(tds, connection->host_name, host_name_len);
 		tds_put_n(tds, domain, domain_len);
 	}
 
