@@ -26,34 +26,37 @@
 #endif /* HAVE_STRING_H */
 
 #include <tds.h>
+#include <tdsconvert.h>
 #include <stdio.h>
+#include <stdlib.h>
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: numeric.c,v 1.15 2002/12/14 15:15:03 freddy77 Exp $";
+static char software_version[] = "$Id: numeric.c,v 1.27 2004/10/14 14:27:50 freddy77 Exp $";
 static void *no_unused_var_warn[] = {
 	software_version,
 	no_unused_var_warn
 };
 
 /* 
-** these routines use arrays of unsigned char to handle arbitrary
-** precision numbers.  All-in-all it's probably pretty slow, but it
-** does work. I just heard of a GNU lib for arb. precision math, so
-** that might be an option in the future.
-*/
-
+ * these routines use arrays of unsigned char to handle arbitrary
+ * precision numbers.  All-in-all it's probably pretty slow, but it
+ * does work. I just heard of a GNU lib for arb. precision math, so
+ * that might be an option in the future.
+ */
+#ifndef HAVE_INT64
 static int multiply_byte(unsigned char *product, int num, unsigned char *multiplier);
 static int do_carry(unsigned char *product);
 static char *array_to_string(unsigned char *array, int scale, char *s);
+#endif
 
 /*
-** The following little table is indexed by precision-1 and will
-** tell us the number of bytes required to store the specified
-** precision (with the sign).
-** Support precision up to 77 digits
-*/
+ * The following little table is indexed by precision-1 and will
+ * tell us the number of bytes required to store the specified
+ * precision (with the sign).
+ * Support precision up to 77 digits
+ */
 const int tds_numeric_bytes_per_prec[] = {
 	-1, 2, 2, 3, 3, 4, 4, 4, 5, 5,
 	6, 6, 6, 7, 7, 8, 8, 9, 9, 9,
@@ -66,47 +69,47 @@ const int tds_numeric_bytes_per_prec[] = {
 };
 
 /*
-** money is a special case of numeric really...that why its here
-*/
+ * money is a special case of numeric really...that why its here
+ */
 char *
 tds_money_to_string(const TDS_MONEY * money, char *s)
 {
-	/* use brian's money */
-#ifdef UseBillsMoney
-	char rawlong[64];
-	int rawlen;
-	int i;
+#ifdef HAVE_INT64
+	int frac;
+	TDS_INT8 mymoney;
+	TDS_UINT8 n;
+	char *p;
 
-	if (mymoney <= -10000 || mymoney >= 10000) {
-#		if (SIZEOF_LONG_LONG > 0)
-		sprintf(rawlong, "%lld", mymoney);
-#		else
-		sprintf(rawlong, "%ld", mymoney);
-#		endif
-		rawlen = strlen(rawlong);
+	/* sometimes money it's only 4-byte aligned so always compute 64-bit */
+	/* FIXME align money/double/bigint in row to 64-bit */
+	mymoney = (((TDS_INT8)(((TDS_INT*)money)[0])) << 32) | ((TDS_UINT*) money)[1];
 
-		strncpy(tmpstr, rawlong, rawlen - 4);
-		tmpstr[rawlen - 4] = '.';
-		strcpy(&tmpstr[rawlen - 3], &rawlong[rawlen - 4]);
+	p = s;
+	if (mymoney < 0) {
+		*p++ = '-';
+		/* we use unsigned cause this cause arithmetic problem for -2^63*/
+		n = -mymoney;
 	} else {
-		i = mymoney;
-		s = tmpstr;
-		if (i < 0) {
-			*s++ = '-';
-			i = -i;
-		}
-		sprintf(s, "0.%04d", i);
+		n = mymoney;
 	}
+	n = (n + 50) / 100;
+	frac = n % 100;
+	n /= 100;
+	/* if machine is 64 bit you do not need to split n */
+#if SIZEOF_LONG < 8
+	if (n >= 1000000000) {
+		sprintf(p, "%ld%09ld.%02d", (long)(n / 1000000000), (long)(n % 1000000000), frac);
+	} else
+#endif
+		sprintf(p, "%ld.%02d", (long)n, frac);
 	return s;
 #else
 	unsigned char multiplier[MAXPRECISION], temp[MAXPRECISION];
 	unsigned char product[MAXPRECISION];
 	const unsigned char *number;
 	unsigned char tmpnumber[8];
-	int num_bytes = 8;
-	int i;
-	int pos;
-	int neg = 0;
+	int i, num_bytes = 8;
+	int pos, neg = 0;
 
 	memset(multiplier, 0, MAXPRECISION);
 	memset(product, 0, MAXPRECISION);
@@ -118,9 +121,10 @@ tds_money_to_string(const TDS_MONEY * money, char *s)
 	/* big endian makes things easy */
 	memcpy(tmpnumber, number, 8);
 #else
-	/* money is two 32 bit ints and thus is out of order on 
-	 * ** little endian machines. Proof of the superiority of 
-	 * ** big endian design. ;)
+	/*
+	 * money is two 32 bit ints and thus is out of order on 
+	 * little endian machines. Proof of the superiority of 
+	 * big endian design. ;)
 	 */
 	for (i = 0; i < 4; i++)
 		tmpnumber[3 - i] = number[i];
@@ -153,38 +157,17 @@ tds_money_to_string(const TDS_MONEY * money, char *s)
 	} else {
 		array_to_string(product, 4, s);
 	}
+
+	/* round to two decimal places */
+	if (s) {
+		sprintf(s, "%.02f", atof(s));
+	}
+
 	return s;
 #endif
 }
 
-char *
-tds_numeric_to_string(const TDS_NUMERIC * numeric, char *s)
-{
-	unsigned char multiplier[MAXPRECISION], temp[MAXPRECISION];
-	unsigned char product[MAXPRECISION];
-	const unsigned char *number;
-	int num_bytes;
-	int pos;
-
-	memset(multiplier, 0, MAXPRECISION);
-	memset(product, 0, MAXPRECISION);
-	multiplier[0] = 1;
-	number = numeric->array;
-	num_bytes = tds_numeric_bytes_per_prec[numeric->precision];
-
-	if (numeric->array[0] == 1)
-		*s++ = '-';
-
-	for (pos = num_bytes - 1; pos > 0; pos--) {
-		multiply_byte(product, number[pos], multiplier);
-
-		memcpy(temp, multiplier, MAXPRECISION);
-		memset(multiplier, 0, MAXPRECISION);
-		multiply_byte(multiplier, 256, temp);
-	}
-	array_to_string(product, numeric->scale, s);
-	return s;
-}
+#ifndef HAVE_INT64
 static int
 multiply_byte(unsigned char *product, int num, unsigned char *multiplier)
 {
@@ -206,6 +189,7 @@ multiply_byte(unsigned char *product, int num, unsigned char *multiplier)
 	}
 	return 0;
 }
+
 static int
 do_carry(unsigned char *product)
 {
@@ -219,6 +203,7 @@ do_carry(unsigned char *product)
 	}
 	return 0;
 }
+
 static char *
 array_to_string(unsigned char *array, int scale, char *s)
 {
@@ -241,3 +226,107 @@ array_to_string(unsigned char *array, int scale, char *s)
 	s[j] = '\0';
 	return s;
 }
+#endif
+
+TDS_INT
+tds_numeric_to_string(const TDS_NUMERIC * numeric, char *s)
+{
+	const unsigned char *number;
+
+	unsigned int packet[sizeof(numeric->array) / 2];
+	unsigned int *pnum, *packet_start;
+	unsigned int *const packet_end = packet + TDS_VECTOR_SIZE(packet);
+
+	unsigned int packet10k[(MAXPRECISION + 3) / 4];
+	unsigned int *p;
+
+	int num_bytes;
+	unsigned int remainder, n, i, m;
+
+	/* a bit of debug */
+#if ENABLE_EXTRA_CHECKS
+	memset(packet, 0x55, sizeof(packet));
+	memset(packet10k, 0x55, sizeof(packet10k));
+#endif
+
+	if (numeric->precision < 1 || numeric->precision > MAXPRECISION || numeric->scale > numeric->precision)
+		return TDS_CONVERT_FAIL;
+
+	/* set sign */
+	if (numeric->array[0] == 1)
+		*s++ = '-';
+
+	/* put number in a 16bit array */
+	number = numeric->array;
+	num_bytes = tds_numeric_bytes_per_prec[numeric->precision];
+
+	n = num_bytes - 1;
+	pnum = packet_end;
+	for (; n > 1; n -= 2)
+		*--pnum = number[n - 1] * 256 + number[n];
+	if (n == 1)
+		*--pnum = number[n];
+	while (!*pnum) {
+		++pnum;
+		if (pnum == packet_end) {
+			*s++ = '0';
+			if (numeric->scale) {
+				*s++ = '.';
+				i = numeric->scale;
+				do {
+					*s++ = '0';
+				} while (--i);
+			}
+			*s++ = 0;
+			return TDS_SUCCEED;
+		}
+	}
+	packet_start = pnum;
+
+	/* transform 2^16 base number in 10^4 base number */
+	for (p = packet10k + TDS_VECTOR_SIZE(packet10k); packet_start != packet_end;) {
+		pnum = packet_start;
+		n = *pnum;
+		remainder = n % 10000u;
+		if (!(*pnum++ = (n / 10000u)))
+			packet_start = pnum;
+		for (; pnum != packet_end; ++pnum) {
+			n = remainder * (256u * 256u) + *pnum;
+			remainder = n % 10000u;
+			*pnum = n / 10000u;
+		}
+		*--p = remainder;
+	}
+
+	/* transform to 10 base number and output */
+	i = 4 * ((packet10k + TDS_VECTOR_SIZE(packet10k)) - p);	/* current digit */
+	/* skip leading zeroes */
+	n = 1000;
+	remainder = *p;
+	while (remainder < n)
+		n /= 10, --i;
+	if (i <= numeric->scale) {
+		*s++ = '0';
+		*s++ = '.';
+		m = i;
+		while (m < numeric->scale)
+			*s++ = '0', ++m;
+	}
+	for (;;) {
+		*s++ = (remainder / n) + '0';
+		--i;
+		remainder %= n;
+		n /= 10;
+		if (!n) {
+			n = 1000;
+			if (++p == packet10k + TDS_VECTOR_SIZE(packet10k))
+				break;
+			remainder = *p;
+		}
+		if (i == numeric->scale)
+			*s++ = '.';
+	}
+	*s++ = 0;
+
+	return TDS_SUCCEED;
+}
