@@ -1,15 +1,14 @@
-#include <stdio.h>
-#include <string.h>
-#include <tds.h>
 #include "common.h"
 
-static char software_version[] = "$Id: common.c,v 1.12 2003/01/27 10:31:29 freddy77 Exp $";
+static char software_version[] = "$Id: common.c,v 1.23 2004/09/08 12:51:25 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 char USER[512];
 char SERVER[512];
 char PASSWORD[512];
 char DATABASE[512];
+/* TODO use another default ?? */
+char CHARSET[512] = "ISO-8859-1";
 
 int read_login_info(void);
 
@@ -42,15 +41,16 @@ read_login_info(void)
 			strcpy(DATABASE, s2);
 		}
 	}
+	fclose(in);
 	return TDS_SUCCEED;
 }
 
+TDSCONTEXT *test_context = NULL;
 
 int
 try_tds_login(TDSLOGIN ** login, TDSSOCKET ** tds, const char *appname, int verbose)
 {
-	TDSCONTEXT *context;
-	TDSCONNECTINFO *connect_info;
+	TDSCONNECTION *connection;
 
 	if (verbose) {
 		fprintf(stdout, "Entered tds_try_login()\n");
@@ -83,26 +83,25 @@ try_tds_login(TDSLOGIN ** login, TDSSOCKET ** tds, const char *appname, int verb
 	tds_set_host(*login, "myhost");
 	tds_set_library(*login, "TDS-Library");
 	tds_set_server(*login, SERVER);
-	tds_set_charset(*login, "iso_1");
+	tds_set_client_charset(*login, CHARSET);
 	tds_set_language(*login, "us_english");
-	tds_set_packet(*login, 512);
 
 	if (verbose) {
 		fprintf(stdout, "Connecting to database\n");
 	}
-	context = tds_alloc_context();
-	*tds = tds_alloc_socket(context, 512);
+	test_context = tds_alloc_context();
+	*tds = tds_alloc_socket(test_context, 512);
 	tds_set_parent(*tds, NULL);
-	connect_info = tds_read_config_info(NULL, *login, context->locale);
-	if (!connect_info || tds_connect(*tds, connect_info) == TDS_FAIL) {
-		if (connect_info) {
+	connection = tds_read_config_info(NULL, *login, test_context->locale);
+	if (!connection || tds_connect(*tds, connection) == TDS_FAIL) {
+		if (connection) {
 			*tds = NULL;
-			tds_free_connect(connect_info);
+			tds_free_connection(connection);
 		}
 		fprintf(stderr, "tds_connect() failed\n");
 		return TDS_FAIL;
 	}
-	tds_free_connect(connect_info);
+	tds_free_connection(connection);
 
 	return TDS_SUCCEED;
 }
@@ -117,6 +116,8 @@ try_tds_logout(TDSLOGIN * login, TDSSOCKET * tds, int verbose)
 	}
 	tds_free_socket(tds);
 	tds_free_login(login);
+	tds_free_context(test_context);
+	test_context = NULL;
 	return TDS_SUCCEED;
 }
 
@@ -133,12 +134,12 @@ run_query(TDSSOCKET * tds, const char *query)
 		return TDS_FAIL;
 	}
 
-	while ((rc = tds_process_result_tokens(tds, &result_type)) == TDS_SUCCEED) {
+	while ((rc = tds_process_result_tokens(tds, &result_type, NULL)) == TDS_SUCCEED) {
 
 		switch (result_type) {
-		case TDS_CMD_DONE:
-		case TDS_CMD_SUCCEED:
-		case TDS_CMD_FAIL:
+		case TDS_DONE_RESULT:
+		case TDS_DONEPROC_RESULT:
+		case TDS_DONEINPROC_RESULT:
 			/* ignore possible spurious result (TDS7+ send it) */
 		case TDS_STATUS_RESULT:
 			break;
