@@ -16,16 +16,9 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-#include <tds.h>
 #include "common.h"
 
-static char software_version[] = "$Id: t0001.c,v 1.5 2002/11/20 13:34:49 freddy77 Exp $";
+static char software_version[] = "$Id: t0001.c,v 1.6 2003/04/21 16:06:10 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 int
