@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,21 +16,9 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
-#include <stdio.h>
-
-#if HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#include <tds.h>
 #include "common.h"
 
-static char software_version[] = "$Id: t0003.c,v 1.6 2002/11/22 15:40:17 freddy77 Exp $";
+static char software_version[] = "$Id: t0003.c,v 1.16 2004/02/03 19:28:12 jklowden Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 
@@ -41,7 +29,6 @@ main(int argc, char **argv)
 	TDSSOCKET *tds;
 	int verbose = 0;
 	int rc;
-	int marker;
 
 	fprintf(stdout, "%s: Testing DB change -- 'use tempdb'\n", __FILE__);
 	rc = try_tds_login(&login, &tds, __FILE__, verbose);
@@ -57,20 +44,23 @@ main(int argc, char **argv)
 	}
 
 	/* warning: this mucks with some internals to get the env chg message */
-	do {
-		marker = tds_get_byte(tds);
-		tds_process_default_tokens(tds, marker);
-	} while (marker != TDS_DONE_TOKEN);
+	if (tds_process_simple_query(tds) != TDS_SUCCEED) {
+		fprintf(stderr, "query results failed\n");
+		return 1;
+	}
 
-	/* Test currently disabled during TDSENVINFO changes */
-	if (tds && tds->env && tds->env->database) {
-		if (verbose) {
-			fprintf(stdout, "database changed to %s\n", tds->env->database);
-		}
-		if (strcmp(tds->env->database, "tempdb")) {
-			fprintf(stderr, "Wrong database, %s != tempdb\n", tds->env->database);
-			return 1;
-		}
+	if (!tds || !tds->env || !tds->env->database) {
+		fprintf(stderr, "No database ??\n");
+		return 1;
+	}
+
+	/* Test currently disabled during TDSENV changes */
+	if (verbose) {
+		fprintf(stdout, "database changed to %s\n", tds->env->database);
+	}
+	if (strcmp(tds->env->database, "tempdb")) {
+		fprintf(stderr, "Wrong database, %s != tempdb\n", tds->env->database);
+		return 1;
 	}
 
 	try_tds_logout(login, tds, verbose);
