@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 1998-1999  Brian Bruns
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,6 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <assert.h>
+
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
@@ -54,13 +56,22 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif /* HAVE_SYS_SELECT_H */
+
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif /* HAVE_SYS_SOCKET_H */
+
 #include "tds.h"
+#include "tdsiconv.h"
 #include <signal.h>		/* GW ADDED */
 #ifdef DMALLOC
 #include <dmalloc.h>
 #endif
 
-static char software_version[] = "$Id: write.c,v 1.31 2002/12/10 17:02:18 freddy77 Exp $";
+static char software_version[] = "$Id: write.c,v 1.66 2004/07/29 10:22:42 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 static int tds_write_packet(TDSSOCKET * tds, unsigned char final);
@@ -76,15 +87,28 @@ static int tds_write_packet(TDSSOCKET * tds, unsigned char final);
 int
 tds_put_n(TDSSOCKET * tds, const void *buf, int n)
 {
-	int i;
+	int left;
 	const unsigned char *bufp = (const unsigned char *) buf;
 
-	if (bufp) {
-		for (i = 0; i < n; i++)
-			tds_put_byte(tds, bufp[i]);
-	} else {
-		for (i = 0; i < n; i++)
-			tds_put_byte(tds, '\0');
+	assert(n >= 0);
+
+	for (; n;) {
+		left = tds->env->block_size - tds->out_pos;
+		if (left <= 0) {
+			tds_write_packet(tds, 0x0);
+			tds_init_write_buf(tds);
+			continue;
+		}
+		if (left > n)
+			left = n;
+		if (bufp) {
+			memcpy(tds->out_buf + tds->out_pos, bufp, left);
+			bufp += left;
+		} else {
+			memset(tds->out_buf + tds->out_pos, 0, left);
+		}
+		tds->out_pos += left;
+		n -= left;
 	}
 	return 0;
 }
@@ -92,33 +116,77 @@ tds_put_n(TDSSOCKET * tds, const void *buf, int n)
 /**
  * Output a string to wire
  * automatic translate string to unicode if needed
+ * @return bytes written to wire
  * @param s   string to write
- * @param len lenth of string or -1 for null terminated
+ * @param len length of string in characters, or -1 for null terminated
  */
 int
 tds_put_string(TDSSOCKET * tds, const char *s, int len)
 {
-	int res;
-	char temp[256];
-	const char *p;
-	unsigned int bytes_left;
+	TDS_ENCODING *client, *server;
+	char outbuf[256], *poutbuf;
+	size_t inbytesleft, outbytesleft, bytes_out = 0;
 
-	if (len < 0)
-		len = strlen(s);
-	if (IS_TDS7_PLUS(tds)) {
-		p = s;
-		res = 0;
-		while (len > 0) {
-			bytes_left = len > (sizeof(temp) / 2) ? (sizeof(temp) / 2) : len;
-			tds7_ascii2unicode(tds, p, temp, bytes_left * 2);
-			res = tds_put_n(tds, temp, bytes_left * 2);
-			p += bytes_left;
-			len -= bytes_left;
+	client = &tds->char_convs[client2ucs2]->client_charset;
+	server = &tds->char_convs[client2ucs2]->server_charset;
+
+	if (len < 0) {
+		if (client->min_bytes_per_char == 1) {	/* ascii or UTF-8 */
+			len = strlen(s);
+		} else if (client->min_bytes_per_char == 2 && client->max_bytes_per_char == 2) {	/* UCS-2 or variant */
+			const char *p = s;
+
+			while (p[0] || p[1])
+				p += 2;
+			len = p - s;
+
+		} else {
+			assert(client->min_bytes_per_char < 3);	/* FIXME */
 		}
-		return res;
-
 	}
-	return tds_put_n(tds, s, len);
+
+	assert(len >= 0);
+
+	/* valid test only if client and server share a character set. TODO conversions for Sybase */
+	if (!IS_TDS7_PLUS(tds))	{
+		tds_put_n(tds, s, len);
+		return len;
+	}
+
+	memset(&tds->char_convs[client2ucs2]->suppress, 0, sizeof(tds->char_convs[client2ucs2]->suppress));
+	tds->char_convs[client2ucs2]->suppress.e2big = 1;
+	inbytesleft = len;
+	while (inbytesleft) {
+		tdsdump_log(TDS_DBG_NETWORK, "tds_put_string converting %d bytes of \"%s\"\n", (int) inbytesleft, s);
+		outbytesleft = sizeof(outbuf);
+		poutbuf = outbuf;
+		
+		if ((size_t)-1 == tds_iconv(tds, tds->char_convs[client2ucs2], to_server, &s, &inbytesleft, &poutbuf, &outbytesleft)){
+		
+			if (errno == EINVAL) {
+				tdsdump_log(TDS_DBG_NETWORK, "tds_put_string: tds_iconv() encountered partial sequence. "
+							     "%d bytes remain.\n", (int) inbytesleft);
+				/* TODO return some sort or error ?? */
+				break;
+			} else if (errno != E2BIG) {
+				/* It's not an incomplete multibyte sequence, or it IS, but we're not anticipating one. */
+				tdsdump_log(TDS_DBG_NETWORK, "Error: tds_put_string: "
+							     "Gave up converting %d bytes due to error %d.\n", 
+							     (int) inbytesleft, errno);
+				tdsdump_dump_buf(TDS_DBG_NETWORK, "Troublesome bytes", s, (int) inbytesleft);
+			}
+
+			if (poutbuf == outbuf) {	/* tds_iconv did not convert anything, avoid infinite loop */
+				tdsdump_log(TDS_DBG_NETWORK, "Error: tds_put_string: No conversion possible, giving up.\n");
+				break;
+			}
+		}
+		
+		bytes_out += poutbuf - outbuf;
+		tds_put_n(tds, outbuf, poutbuf - outbuf);
+	}
+	tdsdump_log(TDS_DBG_NETWORK, "tds_put_string wrote %d bytes\n", (int) bytes_out);
+	return bytes_out;
 }
 
 int
@@ -133,6 +201,29 @@ tds_put_buf(TDSSOCKET * tds, const unsigned char *buf, int dsize, int ssize)
 	return tds_put_byte(tds, cpsize);
 }
 
+int
+tds_put_int8(TDSSOCKET * tds, TDS_INT8 i)
+{
+#if WORDS_BIGENDIAN
+	TDS_UINT h;
+
+	if (tds->emul_little_endian) {
+		h = (TDS_UINT) i;
+		tds_put_byte(tds, h & 0x000000FF);
+		tds_put_byte(tds, (h & 0x0000FF00) >> 8);
+		tds_put_byte(tds, (h & 0x00FF0000) >> 16);
+		tds_put_byte(tds, (h & 0xFF000000) >> 24);
+		h = (TDS_UINT) (i >> 32);
+		tds_put_byte(tds, h & 0x000000FF);
+		tds_put_byte(tds, (h & 0x0000FF00) >> 8);
+		tds_put_byte(tds, (h & 0x00FF0000) >> 16);
+		tds_put_byte(tds, (h & 0xFF000000) >> 24);
+		return 0;
+	}
+#endif
+	return tds_put_n(tds, (const unsigned char *) &i, sizeof(TDS_INT8));
+}
+
 int
 tds_put_int(TDSSOCKET * tds, TDS_INT i)
 {
@@ -161,12 +252,6 @@ tds_put_smallint(TDSSOCKET * tds, TDS_SMALLINT si)
 	return tds_put_n(tds, (const unsigned char *) &si, sizeof(TDS_SMALLINT));
 }
 
-int
-tds_put_tinyint(TDSSOCKET * tds, TDS_TINYINT ti)
-{
-	return tds_put_byte(tds, (unsigned char) ti);
-}
-
 int
 tds_put_byte(TDSSOCKET * tds, unsigned char c)
 {
@@ -178,17 +263,10 @@ tds_put_byte(TDSSOCKET * tds, unsigned char c)
 	return 0;
 }
 
-int
-tds_put_bulk_data(TDSSOCKET * tds, const unsigned char *buf, TDS_INT bufsize)
-{
-
-	tds->out_flag = 0x07;
-	return tds_put_n(tds, buf, bufsize);
-}
-
 int
 tds_init_write_buf(TDSSOCKET * tds)
 {
+	/* TODO needed ?? */
 	memset(tds->out_buf, '\0', tds->env->block_size);
 	tds->out_pos = 8;
 	return 0;
@@ -198,10 +276,10 @@ tds_init_write_buf(TDSSOCKET * tds)
 static int
 tds_check_socket_write(TDSSOCKET * tds)
 {
-int retcode = 0;
-struct timeval selecttimeout;
-time_t start, now;
-fd_set fds;
+	int retcode = 0;
+	struct timeval selecttimeout;
+	time_t start, now;
+	fd_set fds;
 
 	/* Jeffs hack *** START OF NEW CODE */
 	FD_ZERO(&fds);
@@ -214,7 +292,7 @@ fd_set fds;
 			if (retcode >= 0)
 				return 0;
 			/* interrupted */
-			if (sock_errno == EINTR)
+			if (sock_errno == TDSSOCK_EINTR)
 				continue;
 			/* error, leave caller handle problems */
 			return -1;
@@ -228,7 +306,7 @@ fd_set fds;
 		selecttimeout.tv_sec = tds->timeout - (now - start);
 		selecttimeout.tv_usec = 0;
 		retcode = select(tds->s + 1, NULL, &fds, NULL, &selecttimeout);
-		if (retcode < 0 && sock_errno == EINTR) {
+		if (retcode < 0 && sock_errno == TDSSOCK_EINTR) {
 			retcode = 0;
 		}
 
@@ -243,21 +321,31 @@ fd_set fds;
 static int
 goodwrite(TDSSOCKET * tds)
 {
-int left;
-unsigned char *p;
-int result = TDS_SUCCEED;
-int retval;
+	int left;
+	unsigned char *p;
+	int result = TDS_SUCCEED;
+	int retval;
 
 	left = tds->out_pos;
 	p = tds->out_buf;
 
 	while (left > 0) {
-		/* If there's a timeout, we need to sit and wait for socket */
-		/* writability */
-		/* moved socket writability check to own function -- bsb */
+		/*
+		 * If there's a timeout, we need to sit and wait for socket
+		 * writability
+		 * moved socket writability check to own function -- bsb
+		 */
+		/* 
+		 * TODO we can avoid calling select for every send using 
+		 * no-blocking socket... This will reduce syscalls
+		 */
 		tds_check_socket_write(tds);
 
+#ifndef MSG_NOSIGNAL
 		retval = WRITESOCKET(tds->s, p, left);
+#else
+		retval = send(tds->s, p, left, MSG_NOSIGNAL);
+#endif
 
 		if (retval <= 0) {
 			tdsdump_log(TDS_DBG_NETWORK, "TDS: Write failed in tds_write_packet\nError: %d (%s)\n", sock_errno,
@@ -280,7 +368,7 @@ tds_write_packet(TDSSOCKET * tds, unsigned char final)
 {
 	int retcode;
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(MSG_NOSIGNAL)
 	void (*oldsig) (int);
 #endif
 
@@ -292,9 +380,9 @@ tds_write_packet(TDSSOCKET * tds, unsigned char final)
 		tds->out_buf[6] = 0x01;
 	}
 
-	tdsdump_log(TDS_DBG_NETWORK, "Sending packet @ %L\n%D\n", tds->out_buf, tds->out_pos);
+	tdsdump_dump_buf(TDS_DBG_NETWORK, "Sending packet", tds->out_buf, tds->out_pos);
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(MSG_NOSIGNAL)
 	oldsig = signal(SIGPIPE, SIG_IGN);
 	if (oldsig == SIG_ERR) {
 		tdsdump_log(TDS_DBG_WARN, "TDS: Warning: Couldn't set SIGPIPE signal to be ignored\n");
@@ -303,7 +391,7 @@ tds_write_packet(TDSSOCKET * tds, unsigned char final)
 
 	retcode = goodwrite(tds);
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(MSG_NOSIGNAL)
 	if (signal(SIGPIPE, oldsig) == SIG_ERR) {
 		tdsdump_log(TDS_DBG_WARN, "TDS: Warning: Couldn't reset SIGPIPE signal to previous value\n");
 	}
@@ -320,7 +408,7 @@ tds_write_packet(TDSSOCKET * tds, unsigned char final)
 int
 tds_flush_packet(TDSSOCKET * tds)
 {
-int result = TDS_FAIL;
+	int result = TDS_FAIL;
 
 	/* GW added check for tds->s */
 	if (!IS_TDSDEAD(tds)) {
